/*** 
 * \file gre/3dsrtex.h
 * \brief SIMPLERASTER texturing algorithms
 *
 * \if NODOC
 * $Id: 3dsrtex.h_v 1.26 2011/10/20 12:41:40 mju Exp $
 *
 * $Log: 3dsrtex.h_v $
 * Revision 1.26  2011/10/20 12:41:40  mju
 * Remove redundant hdr.
 *
 * Revision 1.25  2010/10/19 21:25:16  mju
 * Add m_useTemporary.
 *
 * Revision 1.24  2007/03/30 17:47:42  mju
 * Use contrastparm.
 *
 * Revision 1.23  2006/10/18 15:25:32  mju
 * Use colorcompstruct* instead of colorcomphandle typedef.
 *
 * Revision 1.22  2006/10/18 14:29:49  mju
 * Use contparm hdr.
 *
 * Revision 1.18  2005/06/27 17:14:48  vdronov
 * added nullmask handling
 *
 * Revision 1.15  2004/07/02 18:02:31  vdronov
 * added UpdateExtents
 *
 * Revision 1.13  2004/06/18 21:11:44  vdronov
 * added relief shading
 *
 * Revision 1.12  2004/06/10 19:20:55  vdronov
 * added m_Opacity;.
 *
 * Revision 1.6  2003/03/31 18:13:37  vdronov
 * nullvalue transparency
 *
 * Revision 1.5  2003/03/31 16:43:29  vdronov
 * multicomponent amd mask support
 *
 * Revision 1.1  2003/03/10 15:07:26  vdronov
 * Initial revision
 * \endif
**/ 

#ifndef	INC_GRE_3DSRTEX_H
#define	INC_GRE_3DSRTEX_H

#ifndef	INC_GRE_3DTEXTUR_H
#include <gre/3dtextur.h>
#endif

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

#ifndef	INC_RVC_NULLMASK_H
#include <rvc/nullmask.h>
#endif

#ifndef	INC_MI32_CONTRASTPARM_H
#include <mi32/contrastparm.h>
#endif

struct COLORCOMPSTRUCT;

namespace GRE {

class SIMPLERASTER : public TEXTURE {
	public:

		SIMPLERASTER (			 							  
			);

		virtual ~SIMPLERASTER (								//! Destructor
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		struct COMPONENT {
			SIMPLE_ARRAY<RVC::RASTER*> m_ImageMap;
			RVC::IMAGE::CONVMODE m_Convert;
			RVC::NULLMASK m_NullMask;
			bool m_DoShading;

			COMPONENT ();

			~COMPONENT ();

			};

		const GRE::LAYER_RASTER *m_Layer;
		DRECT2D ALIGN16(m_Extents);
		SPATREF::COORDREFSYS m_CoordRefSys;
		TRANS2D_MAPGEN m_Transformation;

		COLORCOMPSTRUCT *m_CCHandle;

		RVC::RASTER m_Mask;
		bool m_IsMasked;
		bool m_IsNullTransparent;
		bool m_UseTempRaster;

		double ALIGN8(m_Opacity);

		INT32 m_NumComponents;
		COMPONENT m_Single;
		COMPONENT m_Multiple[4];

		bool m_DoShading;
		CONTRASTPARM m_ContrastParmShade;

		SIMPLERASTER (											//! Copy constructor (not implemented)
			const SIMPLERASTER& rhs
			);

		SIMPLERASTER& operator= (								//! Assignment operator (not implemented)
			const SIMPLERASTER& rhs
			);

		ERRVALUE CreateSingle (
			);

		ERRVALUE CreateMultiple (
			);

		void FreeSingle (
			);

		void FreeMultiple (
			);

		ERRVALUE BeginSingle (
			);

		ERRVALUE BeginMultiple (
			);

		bool GetColorSingle (
			const INT32 line,
			const INT32 column,
			const UINT8 depth,
			const UINT8 mask,
			COLOR& color 
			);

		bool GetColorMultiple (
			const INT32 line,
			const INT32 column,
			const UINT8 depth,
			const UINT8 mask,
			COLOR& color 
			);

		UINT8 CheckDepthSingle (
			const UINT8 depth
			);

		UINT8 CheckDepthMultiple (
			const UINT8 depth
			);

		ERRVALUE UpdateExtents (
			TRANS2D_MAPGEN& transformation, 
			DRECT2D& extents
            );

		TEXTURE* v_CreateInstance (
			) { return new SIMPLERASTER(); };

		ERRVALUE v_Build (
			const GRE::LAYER *layer
			);

		void v_Free (
			);

		void v_Begin (
			const SCENE3D& scene
			);

		void v_End (
			);

		const DRECT2D& v_GetExtents (
			) { return m_Extents; };

		void v_CreateBoundary (
			REGION2D& boundary
			);

		void v_SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			);

		const SPATREF::COORDREFSYS v_GetCoordRefSys (
			) { return m_CoordRefSys; };

		bool v_GetColor (
			const INT32 line,
			const INT32 column,
			const UINT8 depth,
			COLOR& color 
			);

		bool v_GetColor (
			const DPOINT2D& point,
			const UINT8 depth,
			COLOR& color 
			);

		UINT8 v_CheckDepth (
			const UINT8 depth
			);

		const TRANS2D_MAPGEN& v_GetTransformationInternalToModel (
			) { return m_Transformation; };

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		// End of GRE namespace

#endif
