/**
 * \file gre/extrudedparms.h
 * \brief GRE EXTRUDED...PARMS definitions
 *
 * \if NODOC
 * $Id: extrudedparms.h_v 1.1 2006/09/27 16:17:22 mju Exp $
 *
 * $Log: extrudedparms.h_v $
 * Revision 1.1  2006/09/27 16:17:22  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_EXTRUDEDPARMS_H
#define	INC_GRE_EXTRUDEDPARMS_H

#ifndef	INC_GRE_ELEMPARMS_H
	#include <gre/elemparms.h>
#endif

namespace GRE {

//! Parameters for extruded 3d points
struct EXTRUDEDPOINTPARMS {
	public:

		enum MODE {
			MODE_None		= 0,
			MODE_Height	   = 1,
			MODE_Elevation = 2
			};

		MODE Mode;
		RVC::OBJECTNAME TableName;	//!<	table for z value
		RVC::OBJECTNAME FieldName;	//!< field for z value
		UINT8 Unit;					//!<	unit for distance
		LINEPARMS Line;
		POINTPARMS Point;
		bool UseColorStack;
		COLOR BelowColor;			//!<	Color to display wireframe below surface
		COLOR AboveColor;			//!<	Color to display wireframe above surface

		EXTRUDEDPOINTPARMS ();
		EXTRUDEDPOINTPARMS (const EXTRUDEDPOINTPARMS& rhs);
		~EXTRUDEDPOINTPARMS ();

		EXTRUDEDPOINTPARMS& operator= (const EXTRUDEDPOINTPARMS& rhs);
	};

//! Parameters for extruded 3d polygons
struct EXTRUDEDPOLYGONPARMS {
	public:

		enum MODE {
			MODE_None		= 0,
			MODE_Height	   = 1,
			MODE_Elevation = 2
			};
		enum BASE {
			BASE_Minimum	= 0,
			BASE_Maximum	= 1
			};

		MODE Mode;
		BASE Base;
		RVC::OBJECTNAME TableName;			//!<	table for z value
		RVC::OBJECTNAME FieldName;			//!< field for z value
		UINT8 Unit;					//!<	unit for distance
		POLYPARMS Poly;
		bool UseColorStack;
		COLOR BelowColor;			//!<	Color to display wireframe below surface
		COLOR AboveColor;			//!<	Color to display wireframe above surface

		EXTRUDEDPOLYGONPARMS ();
		EXTRUDEDPOLYGONPARMS (const EXTRUDEDPOLYGONPARMS& rhs);
		~EXTRUDEDPOLYGONPARMS ();

		EXTRUDEDPOLYGONPARMS& operator= (const EXTRUDEDPOLYGONPARMS& rhs);
	};

}	// End namespace GRE

#endif	// INC_GRE_EXTRUDEDPARMS_H
