/**
 * \file mgui/listitem.h
 * \brief MGUI LISTITEM definition
 *
 * \if NODOC
 * $Id: listitem.h_v 1.8 2007/01/12 15:41:41 mju Exp $
 *
 * $Log: listitem.h_v $
 * Revision 1.8  2007/01/12 15:41:41  mju
 * Remove listitemdef.
 *
 * Revision 1.7  2006/01/25 14:30:08  mju
 * Deprecate listitemdef.
 *
 * Revision 1.6  2003/10/03 19:58:19  linux32build!build
 * Doxygen
 *
 * Revision 1.5  2003/09/15 13:49:32  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2003/09/10 21:15:20  mju
 * Use MGUI::ID.
 *
 * Revision 1.3  2003/09/10 16:24:24  mju
 * Add LISTITEM class using MISTRING.
 * Add listitemdef::getstring().
 *
 * Revision 1.1  2003/01/16 21:22:49  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_LISTITEM_H
#define	INC_MGUI_LISTITEM_H

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

namespace MGUI {
//===================================================================================================================
//! Structure used in adding multiple items to COMBOBOX and LIST controls.
//! Generally used as a null-terminated array.
class LISTITEM {
	public:

		//! Constructor.
		LISTITEM (
			const MISTRING& string,				//!< String or TEXTID specification
			ID id										//!< ID number
			): m_string(string), m_id(id) { }

		//! Get ID number.
		ID GetID (
			) const { return (m_id); }

		//! Get string.
		const MISTRING& GetString (
			) const { return (m_string); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_string;						//!< String
		ID m_id;										//!< Item ID
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace MGUI

#endif	// INC_MGUI_LISTITEM_H
