/**
 * \file codespace.h <mi32/codespace.h>
 * \brief CODESPACE class defin
 *
 * \if NODOC
 * $Id: codespace.h_v 1.19 2012/06/15 14:42:31 mju Exp $
 *
 * $Log: codespace.h_v $
 * Revision 1.19  2012/06/15 14:42:31  mju
 * Add more asprs_las enums.
 *
 * Revision 1.18  2011/06/22 21:10:07  mju
 * Add asprs.
 *
 * Revision 1.17  2011/06/17 14:20:15  mju
 * Add TOTAL codespace.
 *
 * Revision 1.16  2010/05/17 15:16:51  mju
 * Add fgdc.
 *
 * Revision 1.15  2008/03/27 19:52:28  mju
 * Add codespace_nga.
 *
 * Revision 1.14  2005/11/22 13:45:22  mju
 * Add tydac.
 *
 * Revision 1.13  2005/09/22 17:44:42  mju
 * Add Japan19.
 *
 * Revision 1.12  2005/05/13 16:20:02  mju
 * Export functions from DLL.
 *
 * Revision 1.11  2005/04/20 18:00:28  mju
 * Add gdal and gdal_WKT.
 *
 * Revision 1.10  2005/03/02 14:37:29  mju
 * Add iso_19128_crs.
 *
 * Revision 1.9  2004/10/27 19:17:40  mju
 * Add UTM.
 *
 * Revision 1.8  2004/10/13 14:38:39  mju
 * Add GeoTIFF.
 *
 * Revision 1.7  2004/07/21 15:57:05  mju
 * Add esri_wkt, oracle, oracle_wkt.
 *
 * Revision 1.6  2003/09/26 15:01:29  mju
 * Add MicroImages_Legacy codespace.
 *
 * Revision 1.5  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2003/09/08 20:33:50  dwilliss
 * Added doxygen tags for start/end of function groups
 *
 * Revision 1.3  2003/09/05 13:10:06  mju
 * Rename codespace.
 *
 * Revision 1.2  2003/09/03 14:51:53  mju
 * Add MicroImageSML and PCI codespaces.
 *
 * Revision 1.1  2003/08/18 14:36:52  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_CODESPACE_H
#define	INC_MI32_CODESPACE_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define FUNCLIBEXPORT MI_DLLEXPORT
#else
	#define FUNCLIBEXPORT MI_DLLIMPORT
#endif

//! Enumeration of various codespaces used for translation, identifiers, etc.
// Major codespaces start at multiples of 100 so minor codespaces can be grouped under the major codespace.
enum CODESPACE {
	CODESPACE_MicroImages = 			0,
	CODESPACE_MicroImages_SML =		1,
	CODESPACE_MicroImages_Legacy =	2,
	CODESPACE_Japan19 =					98,	//!< Japan-19 zone IDs (1-19)
	CODESPACE_UTM =						99,	//!< Universal Transverse Mercator zone IDs (1 to 60, -1 to -60 for south)
	CODESPACE_ISO =						100,	//!< International Organization for Standardization
	CODESPACE_ISO_3166_1_A2 =			101,	//!< ISO 3166-1 2-letter alphabetic codes for country names
	CODESPACE_ISO_19128_CRS =			102,	//!< ISO 19128 (Web Map Server) Coordinate Reference System
	CODESPACE_OGC =						200,	//!< OpenGIS Consortium
	CODESPACE_OGC_WKT =					201,	//!< ... Well-Known Text, used for spatial reference systems
	CODESPACE_EPSG =						300,	//!< European Petroleum Survey Group, used for spatial reference system database
	CODESPACE_USGS =						400,	//!< United States Geological Survey
	CODESPACE_USGS_GCTP =				401,	//!< ... General Cartographic Transformation Package
	CODESPACE_NGA =						421,	//!< National Geospatial Intelligence Agency
	CODESPACE_FGDC =						441,	//!< Federal Geograpic Data Committee
	CODESPACE_ESRI =						500,
	CODESPACE_ESRI_WKT =					501,	//!< ESRI Well-Known Text, IDs differ from OGC
	CODESPACE_ERDAS =						600,
	CODESPACE_ERMapper =					700,
	CODESPACE_MapInfo =					800,
	CODESPACE_AutoDesk =					900,
	CODESPACE_PCI =						1000,	//!< PCI Geomatics
	CODESPACE_Oracle =					1100,
	CODESPACE_Oracle_WKT =				1101,	//!< Oracle Well-Known Text, IDs differ from OGC
	CODESPACE_GeoTIFF =					1200,	//!< GeoTIFF (for coordinate transformation methods)
	CODESPACE_GDAL =						1300,	//!< GDAL Library
	CODESPACE_GDAL_WKT =					1301,	//!< GDAL Well-Known Text, some IDs differ from OGC
	CODESPACE_Tydac =						1400,	//!< Tydac
	CODESPACE_TOTAL =						1500,	//!< TOTAL
	CODESPACE_ASPRS =						1600,	//!< ASPRS
	CODESPACE_ASPRS_LAS =				1601,	//!< ASPRS LAS generic
	CODESPACE_ASPRS_LAS_1_0 =			1602,	//!< ASPRS LAS 1.0 - 1.3 (Lidar)
	CODESPACE_ASPRS_LAS_1_4 =			1603,	//!< ASPRS LAS 1.4 (Lidar)
	CODESPACE_Unknown =					-1
	};

//!:Associate with "Codespace Functions"
//!\addtogroup Codespace Codespace Functions
//!@{

//! Get CODESPACE for given codespace name.
//! @return Codespace enumeration value, CODESPACE_Unknown if name not valid.
FUNCLIBEXPORT CODESPACE CodespaceFromName (
	const char *name							//!< Codespace name
	);

//! Get name string for specified codespace.
//! Name strings are used in various XML files.
//! @return Codespace name
FUNCLIBEXPORT const char *CodespaceGetName (
	CODESPACE codespace						//!< Codespace
	);

//!@}

#undef	FUNCLIBEXPORT

#endif	//!< INC_MI32_CODESPACE_H
