/**
 * \file mi32/genericrwmem.h
 * \brief GENERICRW_MEMORY class definitions.
 *
 * \if NODOC
 * $Id: genericrwmem.h_v 1.8 2011/12/28 22:13:25 mju Exp $
 *
 * $Log: genericrwmem.h_v $
 * Revision 1.8  2011/12/28 22:13:25  mju
 * Remove redundant hdr.
 *
 * Revision 1.7  2006/07/07 22:49:36  scowan
 * Make dtor virtual.
 *
 * Revision 1.1  2003/07/11 15:25:34  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_GENERICRWMEM_H
#define	INC_MI32_GENERICRWMEM_H
#define	INC_MI32_GENERICRWMEM_H

#ifndef	INC_MI32_GENERICRW_H
	#include <mi32/genericrw.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//===================================================================================================================

//! Memory buffer supporting generic read/write interface.
class CLASSLIBEXPORT GENERICRW_MEMORY : public GENERICRW {
	public:

		//! Constructor.
		GENERICRW_MEMORY ();

		//! Destructor.
		virtual ~GENERICRW_MEMORY ();

		//! Attach allocated array memory buffer.
		void Attach (
			UINT8 *& items,							//!< Buffer to attach
			int numitems								//!< Number of items in array
			);

		//! Attach SIMPLE_ARRAY<UINT8> to instance.
		void Attach (
			SIMPLE_ARRAY<UINT8>& buffer			//!< Buffer to attach, will be detached from here
			);

		//! Detach memory buffer.
		//! Return pointer to buffer, caller must free.
		UINT8* Detach (
			);

		//! Get memory buffer.
		const SIMPLE_ARRAY<UINT8>& GetBuffer (
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<UINT8> m_buffer;
		UINT32 m_curpos;

		//! Overrides from GENERICRW

		virtual CAPABILITIES v_GetCapabilities () const;
		virtual void* v_GetMappedMemory(INT64& size, INT64 offset);
		virtual INT64 v_GetSize () const;
		virtual INT64 v_GetSizeLimit () const;
		virtual INT32 v_Read (void* buffer, INT32 numbytes, bool AllowPartial);
		virtual ERRVALUE v_Resize (INT64 newsize);
		virtual INT64 v_Seek (INT64 offset, SEEKFROM from);
		virtual INT64 v_Tell () const;
		virtual ERRVALUE v_Write (const void* buffer, INT32 numbytes);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

#undef	CLASSLIBEXPORT

#endif	//!< INC_MI32_GENERICRWMEM_H
