/**
 * \file mi32/inihnd.h
 * \brief INIHND class definition
 *
 * \if NODOC
 * $Id: inihnd.h_v 1.3 2007/09/05 22:13:15 scowan Exp $
 *
 * $Log: inihnd.h_v $
 * Revision 1.3  2007/09/05 22:13:15  scowan
 * Readded necessary filepath include file.
 *
 * Revision 1.2  2006/10/09 20:09:37  mju
 * Fwd-declare filepath instead of using header.
 *
 * Revision 1.1  2006/09/25 16:41:21  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_INIHND_H
#define	INC_MI32_INIHND_H

#ifndef	INC_MI32_INIDEFNS_H
	#include <mi32/inidefns.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
	#include <mi32/filepath.h>		// Since Open() method calls FILEPATH::operator==(), need the include file.
#endif

//! INI handle class support automatic closing of handle when going out of scope.
class INIHND {
	public:

		//! Construct unopened handle.
		INIHND (
			): m_handle(0) { }

		//! Destructor, closes handle if open.
		~INIHND (
			) { Close(); }

		//! Cast to INIHANDLE - Return actual handle for use in other Ini functions.
		//! Do not use in IniClose or errors will occur.
		operator INIHANDLE (
			) const { return (m_handle); }

		//! Close handle if open.
		void Close (
			) { if (IsOpen()) IniClose(m_handle); m_handle = 0; }

		//! Determine if handle references 'open' file.
		bool IsOpen (
			) const { return (m_handle != 0); }

		//! Open "tnthost.ini" file.
		ERRVALUE OpenHost (
			bool readonly = true
			) { Close(); return (IniOpen(INIFILE_Host,&m_handle,(readonly?INIFLAG_ReadOnly:0))); }

		//! Open specified file given FILEPATH.
		ERRVALUE Open (
			const FILEPATH& filepath
			) {
			if (IsOpen() && filepath == IniGetFilePath(m_handle)) return (0);
			Close();
			return (IniOpen(filepath,&m_handle));
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INIHANDLE m_handle;
		// UNIMPLEMENTED / UNSUPPORTED
		INIHND (const INIHND&);
		INIHND& operator= (const INIHND&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//-------------------------------------------------------------------------------------------------


#endif	// INC_MI32_INIHND_H
