/**
 * \file mi32/lidarlasrecordhistogram.h
 * \brief LIDAR::LAS::RECORD_HISTOGRAM definitions
 *
 * \if NODOC
 * $Id: lidarlasrecordhistogram.h_v 1.1 2012/09/07 20:26:07 mju Exp $
 *
 * $Log: lidarlasrecordhistogram.h_v $
 * Revision 1.1  2012/09/07 20:26:07  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_LIDARLASRECORDHISTOGRAM_H
#define	INC_MI32_LIDARLASRECORDHISTOGRAM_H

#ifndef	INC_MI32_LIDARLASRECORD_H
#include <mi32/lidarlasrecord.h>
#endif

#ifndef	INC_MI32_LIDARLASFILE_H
#include <mi32/lidarlasfile.h>
#endif

#ifndef	INC_MI32_STATUNIVARIATE_H
#include <mi32/statunivar.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
class RECORDBUFFER_LSBFIRST;
#endif


#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


namespace LIDAR {
namespace LAS {

//=====================================================================================================================

//! Base for "histogram" records.
class CLASSLIBEXPORT RECORD_HISTOGRAM : public RECORD_SCAN {
	public:

		//! Destructor.
		virtual ~RECORD_HISTOGRAM ();

		const STAT_UNIVARIATE& GetStats () const
			{ return (m_Stats); }

	protected:

		//! Constructor.
		RECORD_HISTOGRAM (
			USERID userid,
			RECORDID recordid,
			DATATYPE datatype
			);

		//! Access statistics for initialize/accumulate.
		STAT_UNIVARIATE& AccessStats ()
			{ return (m_Stats); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		STAT_UNIVARIATE m_Stats;
		USERID m_userid;
		RECORDID m_recordid;
		DATATYPE m_datatype;

		int GetAnyType (RECORDBUFFER_LSBFIRST& buffer, int offset, DATATYPE datatype, double& value) const;
		int PutAnyType (RECORDBUFFER_LSBFIRST& buffer, int offset, double value) const;

		// RECORD overrides.
		virtual int v_Read (LAS::FILE& file);
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const;
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_Finalize ();

		#endif
	};

//=====================================================================================================================

class CLASSLIBEXPORT RECORD_HISTOGRAM_INTENSITY : public RECORD_HISTOGRAM {
	public:

		//! Constructor.
		RECORD_HISTOGRAM_INTENSITY ();

		//! Destructor.
		virtual ~RECORD_HISTOGRAM_INTENSITY ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& PointRec);
		virtual ERRVALUE v_Initialize (LAS::FILE& file);
		#endif
	};

//=====================================================================================================================

class CLASSLIBEXPORT RECORD_HISTOGRAM_RAWZ : public RECORD_HISTOGRAM {
	public:

		//! Constructor.
		RECORD_HISTOGRAM_RAWZ ();

		//! Destructor.
		virtual ~RECORD_HISTOGRAM_RAWZ ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& PointRec);
		virtual ERRVALUE v_Initialize (LAS::FILE& file);
		#endif
	};

//=====================================================================================================================

}	// End namespace LAS
}	// End namespace LIDAR

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_LIDARLASRECORDHISTOGRAM_H
