#ifdef WIN32
#pragma message(__FILE__ "(2) : warning C9999: Use of mstatus.h is deprecated, use mgui/dlgstatus.h or mi32/mistatus.h instead.")
#endif

/**
 * \file mi32/mstatus.h
 * \brief Include file for "status display" functions
 *
 * \if NODOC
 * $Id: mstatus.h_v 1.90 2012/01/10 16:41:18 mju Exp $
 *
 * $Log: mstatus.h_v $
 * Revision 1.90  2012/01/10 16:41:18  mju
 * Add deprecation warning for file.
 *
 * Revision 1.89  2012/01/09 14:44:11  mju
 * Remove unused typedef.
 *
 * Revision 1.88  2011/09/06 21:20:43  mju
 * Exclude macros from dox.
 *
 * Revision 1.84  2007/12/05 15:11:09  mju
 * In barIncrement/Init/Update don't do redundant cancel check.
 *
 * Revision 1.81  2007/11/09 18:32:14  mju
 * Update to mistatusmanager/context classes, all here is now deprecated.
 *
 * Revision 1.79  2007/05/09 19:51:16  mju
 * Mark legacy timer update fn ptr as removed.
 *
 * Revision 1.63  2003/10/10 15:21:26  mju
 * Add ReportDone and UpdateTimer methods using textid.
 *
 * Revision 1.59  2003/08/13 20:18:45  scowan
 * Added bool value to disable cancel class.
 *
 * Revision 1.55  2003/05/21 14:40:22  dwilliss
 * Added STATUSDIALOG::SetSelfAsDefault()
 *
 * Revision 1.44  2002/02/20 18:06:07  mju
 * In CheckCancel check if context null before dereference.
 *
 * Revision 1.41  2001/05/15 14:28:50  scowan
 * Added disable cancel class.
 *
 * Revision 1.40  2001/02/07 16:34:37  scowan
 * Check for status context == 0 defore enable / disable cancel button.
 *
 * Revision 1.39  2001/02/02 22:11:35  polson
 * changed ~STATUSDIALOG to check if already destroyed
 *
 * Revision 1.38  2000/11/20 17:20:54  scowan
 * Added a get handle method for legacy functions.
 *
 * Revision 1.37  2000/10/23 15:06:54  mju
 * Add STATUSDIALOG class.
 *
 * Revision 1.33  2000/06/01 16:48:02  scowan
 * Added capability to create a status context instance without creating a context
 * untill later with a create method.
 *
 * Revision 1.20  1997/11/19  17:11:44  mju
 * Add MstatusTextUpdateFV().
 *
 * Revision 1.19  1997/10/17  16:15:34  mju
 * Change to use MTIMER.
 * Add MstatusTimerGet and MstatusTimerReset.
 *
 * Revision 1.18  1997/10/06  16:50:11  mju
 * Change prototype for MstatusTextUpdateTimer().
 *
 * Revision 1.17  1997/10/03  20:01:07  mju
 * Add MstatusTextUpdateString().
 *
 * Revision 1.16  1997/08/28  20:07:51  dwilliss
 * Added a field to STATUSLINEDATA
 *
 * Revision 1.15  1997/05/23  21:24:03  dwilliss
 * Added support for MSTATUS_DelayPopup
 *
 * Revision 1.14  1997/01/03  16:57:05  mju
 * Change MstatusClearBar to MstatusBarClear.  ditto for Text.
 *
 * Revision 1.12  1996/12/05  16:37:50  dwilliss
 * Added mstatusForm functions
 *
 * Revision 1.11  1996/12/03  18:34:58  mju
 * Add MstatusWaitForOK() and MstatusDialogGetForm().
 *
 * Revision 1.1  1994/08/10  14:49:25  mju
 * Initial revision
 * \endif
**/

#if !defined(INC_MI32_MSTATUS_H) && !defined(NODLLLINKAGE)
#define	INC_MI32_MSTATUS_H

#ifndef	INC_STDARG_H
	#include <stdarg.h>
	#define	INC_STDARG_H
#endif

#ifndef INC_MI32_MISTATUS_H
#include <mi32/mistatus.h>
#endif

#ifndef	INC_MGUI_DLGSTATUS_H
#include <mgui/dlgstatus.h>
#endif


#ifdef WIN32
#pragma warning (push)
#pragma warning (disable:4100)	// Disable warning that this needs a DLL interface
#pragma warning (disable:4996)	// Disable warning about deprecated declaration within header
#endif


//=====================================================================================================================

#ifdef USE_MGUI
//! Status dialog (DEPRECATED).
//! \deprecated Use MGUI::DLG_STATUS.
DEPRECATED class STATUSDIALOG {
	public:

		enum FLAGS {
			FLAG_NoCancel =		MGUI::DLG_STATUS::DLGSTYLE_DisableCancel,
			FLAG_DelayPopup =		MGUI::DLG_STATUS::DLGSTYLE_DelayPopup,
			FLAG_None =				MGUI::DLG_STATUS::DLGSTYLE_Default
			};

		//! Constructor, performs initialization only, does not create dialog itself.
		STATUSDIALOG (
			): m_pDlgStatus(0) { }

		//! Destructor, will destroy dialog if exists.
		~STATUSDIALOG (
			) { delete m_pDlgStatus; }

		//! Check if 'Cancel' button has been pressed.
		//! @return EUserCancel if pressed or 0 if not.
		ERRVALUE CheckCancel (
			) const { return ((m_pDlgStatus != 0 && m_pDlgStatus->IsCancelled()) ? EUserCancel : 0); }

		//! Create the actual dialog.
		ERRVALUE Create (
			MDLGPARENT dlgparent,					//!< Parent for dialog
			FLAGS flags = FLAG_None
			) { m_pDlgStatus = new MGUI::DLG_STATUS; m_pDlgStatus->Create(dlgparent,static_cast<MGUI::DLG_STATUS::DLGSTYLE>(flags)); return (0); }

		//! Destroy dialog without destroying class instance.
		void Destroy (
			) { delete m_pDlgStatus; m_pDlgStatus = 0; }

		//! Disable the 'Cancel' button on the dialog (if any).
		void DisableCancel (
			) const { if (m_pDlgStatus != 0) m_pDlgStatus->SetCancelEnabled(false); }

		//! Enable the 'Cancel' button on the dialog (if any).
		void EnableCancel (
			) const { if (m_pDlgStatus != 0) m_pDlgStatus->SetCancelEnabled(true); }

		//! Get the MISTATUSMANAGER* for the dialog.
		//! \deprecated No new code should be written to take an MISTATUSMANAGER*
		//!	directly, however this method is still necessary for calling some
		//!	old code.
		MISTATUSMANAGER* GetHandle (
			) const { return (m_pDlgStatus); }

		//! Get timer associated with status dialog.
		MTIMER* GetTimer (
			) { return (&m_pDlgStatus->GetTimer()); }

		//! Report that process is finished with time.
		//! This will change the 'Cancel' button to 'OK' wait for the user to press.
		//! If the TEXTID__None is specified a generic 'time to process' message will be shown.
		void ReportDone (
			TEXTID textid = TEXTID__None
			) const { if (m_pDlgStatus != 0) m_pDlgStatus->ReportDone(textid); }

		//! Reset the 'Cancel' button on the dialog (if any).
		void ResetCancel (
			) const { if (m_pDlgStatus != 0) m_pDlgStatus->ResetCancelled(); }

		//! Sets this dialog to be the default status handle.  
		//! Useful when calling functions that don't play nice and set the
		//! default back to 0 when they exit.
		MISTATUSMANAGER* SetSelfAsDefault (
			) { return (MISTATUSMANAGER::SetActiveManager(m_pDlgStatus)); }

		//! Wait for user to press 'OK' button.
		void WaitForOK (
			) const { /*if (m_pDlgStatus != 0) m_pDlgStatus->WaitForClose();*/ }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MGUI::DLG_STATUS *m_pDlgStatus;

		STATUSDIALOG (const STATUSDIALOG&);
		STATUSDIALOG& operator= (const STATUSDIALOG&);

		friend class STATUSCONTEXT;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(STATUSDIALOG::FLAGS)
#endif
#endif

//=====================================================================================================================

//! Status context (DEPRECATED).
//! \deprecated use MISTATUSCONTEXT instead.
DEPRECATED class STATUSCONTEXT {
	public:

		//! Constructor using default status dialog.
		explicit STATUSCONTEXT (				
			MISTATUSMANAGER *pStatusMgr = 0,
			bool CreateContext = true
			) :
			m_pStatusContext(0)
			{ if (CreateContext) Create(pStatusMgr); }

	#ifdef USE_MGUI
		//! Construct context in STATUSDIALOG.
		explicit STATUSCONTEXT (
			STATUSDIALOG& sdialog
			):
			m_pStatusContext(0)
			{ if (sdialog.GetHandle() != 0) Create(sdialog.GetHandle()); }
	#endif

		//! Destructor.
		~STATUSCONTEXT (							
			) { Destroy(); }

		//! Clear the status bar.
		void BarClear (
			) { if (m_pStatusContext != 0) m_pStatusContext->ReportProgress(0); }

		//! Increment progress indicator bar position.
		int BarIncrement (
			INT32 increment = 1,					//!< Amount to increment by 
			int line = 0 // line number no longer supported
			) {
			if (m_pStatusContext == 0) return (0);
			return (m_pStatusContext->ReportProgressIncrement(increment));
			}

		//! Initialize or reset progress bar.
		int BarInit (
			INT32 maximum,							//!< Maximum expected number of calls to BarIncrement()
			int line = 0 // line number no longer supported
			) {
			if (m_pStatusContext == 0) return (0);
			return (m_pStatusContext->ReportProgressBegin(static_cast<UINT32>(maximum)));
			}
				
		//! Update progress indicator bar position. 
		int BarUpdate (
			INT32 value,							//!< Current bar position
			INT32 maximum,							//!< Maximum bar position
			int line = 0 // line number no longer supported
			) {
			if (m_pStatusContext == 0) return (0);
			return (m_pStatusContext->ReportProgress(100.0*value/maximum));
			}

		//! Check if cancel button has been pressed.
		int CheckCancel (
			) { return ((m_pStatusContext != 0 && m_pStatusContext->IsCancelled()) ? EUserCancel : 0); }

		//! Clear all lines in status context.
		void Clear (
			) { if (m_pStatusContext != 0) { m_pStatusContext->ReportProgress(0); m_pStatusContext->ReportMessage(TEXTID__None); } }

		//! Clear both text and indicator bar in context.
		void ClearLine (
			) { Clear(); }
			
		//! Clear both text and indicator bar in context.
		void ClearLine (
			int // line number no longer supported
			) { Clear(); }
			
		//! Create a status context.
		void Create (
			MISTATUSMANAGER* pStatusMgr = 0
			) { m_pStatusContext = (pStatusMgr != 0) ? new MISTATUSCONTEXT(pStatusMgr,TEXTID__None) : new MISTATUSCONTEXT(TEXTID__None); }

		//! Destroy status context.
		//! Automatically called by destructor.
		void Destroy (
			) { delete m_pStatusContext; m_pStatusContext = 0; }

	#ifndef NO_DEPRECATED
		//! Disable cancel button.
		void DisableCancel (
			) { }
	#endif

	#ifndef NO_DEPRECATED
		//! Enable cancel button.
		void EnableCancel (
			) { }
	#endif

	#ifndef NO_DEPRECATED
		//! Obtain internal handle for legacy functions		
		MISTATUSCONTEXT *GetHandle (
			) const { return (m_pStatusContext); }
	#endif

		//! Update text line using a string.
		int TextUpdate (
			const MISTRING& string,					//!< Text string (also implies TEXTID)
			int line = 0 // line number no longer supported
			) {
			if (m_pStatusContext == 0) return (0);
			if (m_pStatusContext->IsCancelled()) return (EUserCancel);
			m_pStatusContext->ReportMessage(string);
			return (0);
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTATUSCONTEXT *m_pStatusContext;

		STATUSCONTEXT (const STATUSCONTEXT&);
		STATUSCONTEXT& operator= (const STATUSCONTEXT&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================

//! Class to disable status "Cancel" button when an instance is created and reenable the button when the instance is destroyed	
//! \deprecated Done automatically by MISTATUSCONTEXT
DEPRECATED class STATUSDISABLECANCEL {
	public:

		//! Constructor.
		STATUSDISABLECANCEL (
			MISTATUSMANAGER* pStatusMgr = 0,
			bool Enable = true
			) :
			m_pStatusContext(0)
			{
			if (Enable) {
				if (pStatusMgr == 0) pStatusMgr = MISTATUSMANAGER::GetActiveManager();
				if (pStatusMgr != 0) m_pStatusContext = new MISTATUSCONTEXT(pStatusMgr,TEXTID__None,false);
				}
			}

		//! Destructor.
		~STATUSDISABLECANCEL (
			) { delete m_pStatusContext; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTATUSCONTEXT *m_pStatusContext;

		STATUSDISABLECANCEL (const STATUSDISABLECANCEL&);
		STATUSDISABLECANCEL& operator=(const STATUSDISABLECANCEL&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

#ifdef WIN32
#pragma warning (pop)
#endif

#endif	// INC_MI32_MSTATUS_H 
