/****************************************************************************
 * \file mi32/ports.h
 * \brief MicroImages Port library
 *
 *===========================================================================
 *
 * \if NODOC
 * $Id: ports.h_v 1.58 2012/01/06 18:01:36 mju Exp $
 *
 * $Log: ports.h_v $
 * Revision 1.58  2012/01/06 18:01:36  mju
 * Remove redundant hdr.
 *
 * Revision 1.57  2007/03/22 22:13:52  dwilliss
 * Added optional parm to PORT::Open
 *
 * Revision 1.56  2007/03/22 21:39:05  dwilliss
 * Moved all the mserial stuff to a private include file.  Only used
 * internally
 *
 * Revision 1.55  2007/03/22 21:06:53  dwilliss
 * return type of v_Write didn't match base class when compiled for Unix
 *
 * Revision 1.52  2007/03/22 17:36:07  dwilliss
 * deleted unused functions and change Mp functions to C++ linkage
 * Added an MpOpen that takes MISTRING
 * Created PORT class derived from GENERICRW
 *
 * Revision 1.51  2007/03/05 20:47:09  bcolombe
 * Changed MPORTTYPE scan back to previous version
 * Changing libmi/portcom.c scan_port to always scan
 *
 * Revision 1.49  2006/09/25 13:30:51  mju
 * Remove scsi stuff.
 *
 * Revision 1.48  2006/04/05 13:15:23  mju
 * Use ptrint for port handles so works on win64.
 *
 * Revision 1.47  2006/02/16 16:37:18  dwilliss
 * changed an ifdef to use a new define instead of a deprecated one
 *
 * Revision 1.43  2002/08/01 22:17:59  dwilliss
 * Added PORTMODE_NonBlocking
 *
 * Revision 1.42  2002/07/19 14:26:29  scowan
 * Made port name a fixed string.
 *
 * Revision 1.37  1999/08/17 22:29:51  dwilliss
 * Added a port type code
 *
 * Revision 1.35  1999/04/02  20:52:39  mju
 * Add MxPortGetSettings() prototype.
 *
 * Revision 1.13  1994/08/30  21:44:17  mju
 * Add InputQueueSize and OutputQueueSize to Port structure.
 *
 * Revision 1.1  1992/12/11  23:49:53  mju
 * Initial revision
 * \endif
****************************************************************************/

#ifndef INC_MI32_PORTS_H
#define INC_MI32_PORTS_H

#ifndef	INC_STDIO_H
	#include <stdio.h>
	#define	INC_STDIO_H
#endif

#ifndef	INC_MI32_FIXEDSTR_H
	#include <mi32/fixedstr.h>
#endif

#ifndef	INC_MI32_GENERICRW_H
	#include <mi32/genericrw.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
struct _PortSpecific {
	FIXEDSTRING<80> name;
	int num;
	};

struct _PortOption {
	char *name;									//!<  Name looked up in resource file 
	int  value;
	};

struct _PortOptionType {
	char *name;									//!<  ingored if radio buttons 
	short num;
	short radio;								//!<  1 = use radio buttons, 0 = use option menu 
	char  *dft;
	_PortOption *list;
	};
#endif //!< GENERATING_DOXYGEN_OUTPUT

struct MPORT;									//!<  Forward declaration to prevent annoying warnings from compiler 

struct MPORTTYPE {
	char name[20];
	int type_num;
	int (*init)(MPORT *, MPORTTYPE *, char *);
	int (*get_status)(MPORT *, char **);
	int (*read_data)(MPORT *, UINT8 *, int);
	int (*write_data)(MPORT *, const UINT8 *, int);
	int (*xxinterface_cmd)(MPORT *, const UINT8 *, int);
	int (*xxset_mode)(MPORT *, short);
	int (*set_parms)(MPORT *, char **);
	int (*xxreset)(MPORT *);
	int (*xxabort)(MPORT *);
	int (*close)(MPORT *);
	int (*scan)(MPORTTYPE *);				/* code to fill in Stuff below */
	int (*flush)(MPORT *);
/*-------------------------------------Filled in later (by scan function)---*/
	short NumPorts;							//!<  If -1, name is user specified file 
	short NumOptionTypes;
	_PortSpecific *specific;	
	_PortOptionType *options;
	};

struct PortChannel {
	char *buf;
	unsigned short len;
	unsigned short received;
	int allocsize;
	int spare;
	};

#define MAXPORTCHANNEL 2					//!<  Channels 0 - 2 

struct MPORT {
	MIUNICODE *name;
	const MPORTTYPE *type;
	int unit_num;								//!<  1 = LPT1... etc... 
	unsigned short mode;						//!<  See below 
	short bogus1;
	void *data;									//!<  module specific data 
	int addr;									//!<  base address of port 
	short word_dma;
	unsigned char intnum;	
	unsigned char dmanum;
	PTRINT out_port;								//!<  port for sending 
	PTRINT in_port;								//!<  port for reading 
	PTRINT secondary_out;
	PTRINT secondary_in;
	FILE *stream;								//!<  For fopen/popen'd files 
	unsigned short status;					//!<  see below 
	UINT8 channel_num;
	UINT8 state;
	PortChannel channels[MAXPORTCHANNEL + 1];
	PortChannel *cp;
	void (*oldintfunc)();
	unsigned char *buf[2], *bufp;
	const unsigned char *busy_data;		//!<  data waiting because dev was busy 
	int busy_len;
	int buflen;
	int bufleft;
	int curpos;
	int curbuf;	
	int lasterr;
	unsigned char out_mask;					//!<  Mask to apply before writing data 
	unsigned char in_mask;					//!<  Mask to apply after reading data 
	int InputQueueSize;						//!<  Size used when port was opened (currently only used by COM ports) 
	int OutputQueueSize;						//!<  Size used when port was opened (currently only used by COM ports) 
	MIUNICODE *message;							//!<  Last error message 
	int	NoWait;								//!<  When reading, don't keep trying if there's nothing to read 
	};

/* The "data" field may be used by the module for the given port type */
/* to store any data it needs which is not already in the structure */
/* It is automatically free()d by MpClose just before it frees the */
/* whole Port structure */

#define PORTTYPE_LPT				0x0001
#define PORTTYPE_COM				0x0002
#define PORTTYPE_FILE			0x0008	
#define PORTTYPE_PIPE			0x0080	//!<  via popen() 
#define PORTTYPE_SPOOLER		0x0100	//!<  lp or lpr using popen() 
#define PORTTYPE_WINPRINT		0x0400	//!<  Windows printer driver 
#define PORTTYPE_MACBACK		0x0800	//!<  Macintosh PrGeneral back door hack 


#define PORTSTAT_Error			0x0008
#define PORTSTAT_OffLine		0x0010
#define PORTSTAT_OutOfPaper	0x0020
#define PORTSTAT_NotReady		0x0080
#define PORTSTAT_TimeOut		0x0001
#define PORTSTAT_Abort			0x8000
#define PORTSTAT_DiskFull		0x0100	//!<  for files only 


//#define PORTMODE_DoubleBuf		0x0001
//#define PORTMODE_SingleBuf		0x0002
//#define PORTMODE_Buffered		0x0003	//!<  Single or double 
//#define PORTMODE_HighSpeed		0x0004	//!<  else via DOS (obsolete)
//#define PORTMODE_SingleStream 0x0020	//!<  Single stream has command/data 
#define PORTMODE_UnicodeName	0x0100	//!<  Name is Unicode 
#define PORTMODE_NonBlocking	0x0200	//!<  Do not block on read 
#define PORTMODE_NOT				0x8000	//!<  Clear the given flags 


#define MSERIALFLOW_Hardware	0
#define MSERIALFLOW_XonXoff	1
#define MSERIALFLOW_None		2
#define MSERIALFLOW_NoCTSDSR	3

#define MSERIALPARITY_None		0
#define MSERIALPARITY_Odd		1
#define MSERIALPARITY_Even		2

#define MSERIALBAUD_300			300
#define MSERIALBAUD_600			600
#define MSERIALBAUD_1200		1200
#define MSERIALBAUD_2400		2400
#define MSERIALBAUD_4800		4800
#define MSERIALBAUD_9600		9600
#define MSERIALBAUD_19_2K		19200
#define MSERIALBAUD_38_4K		28400
#define MSERIALBAUD_57_6K		57600

#define MSERIALDATABITS_7			7
#define MSERIALDATABITS_8			8

#define MSERIALSTOPBITS_1			1
#define MSERIALSTOPBITS_2			2
#define MSERIALSTOPBITS_1_5		3


class PORT : public GENERICRW {
	public:

		enum STATUS {
			STATUS_OK				= 0,
			STATUS_Error			= 0x0008,
			STATUS_OffLine		 	= 0x0010,
			STATUS_OutOfPaper	 	= 0x0020,
			STATUS_NotReady		= 0x0080,
			STATUS_TimeOut		 	= 0x0001,
			STATUS_Abort			= 0x8000,
			STATUS_DiskFull		= 0x0100	//!<  for files only 
			};

		enum TYPE {
			TYPE_Automatic			   = 0,			//! Will determine type based on name
			TYPE_Parallel				= 0x0001,
			TYPE_Serial				 	= 0x0002,
			TYPE_File			 		= 0x0008,	
			TYPE_Pipe					= 0x0080,	//!<  via popen() 
			TYPE_WindowPrinter		= 0x0400,	//!<  Windows printer driver 
			};

		//! Constructor
		PORT (
			TYPE type = TYPE_Automatic
			);

		//! Construct a port given a port type name.
		//! This one is used by SML. Port type names are like
		//! 	"serial_port", "parallel_port", etc.
		//! Also used when the port type needs to be stored in an ini file
		PORT(const char* PortType);

		//! Construct a port given an MPORTTYPE*.
		//! This version will eventually go away.  It's just here as a temporary
		//! link betweek old code and new code
		PORT(const MPORTTYPE* PortType);

		virtual ~PORT();

		ERRVALUE Flush (
			);

		STATUS GetStatus (
			) const;

		//! Determine if a serial port has data available
		//! Returns false for non-serial ports
		bool HasDataAvailable (
			) const;

		ERRVALUE Open (
			const MISTRING& portname,
			const char* parameters = 0,
			bool bNonBlocking = false
			);

		void SetParms (
			const char*
			);

	public:
		const MPORTTYPE* m_PortType;
		MPORT* m_priv;

		virtual CAPABILITIES v_GetCapabilities () const;
		virtual INT64 v_GetSize () const;
		virtual INT64 v_GetSizeLimit () const;
		virtual INT32 v_Read (void* buffer, INT32 numbytes, bool AllowPartial);
		virtual ERRVALUE v_Resize (INT64 newsize);
		virtual INT64 v_Seek (INT64 offset, SEEKFROM from);
		virtual INT64 v_Tell () const;
		virtual ERRVALUE v_Write (const void* buffer, INT32 numbytes);
	};


/*-------------------------------------------------------------------------*/
/*		Function Prototypes																	*/
/*-------------------------------------------------------------------------*/

//! Close a port.
//!
//! This routine closes the port and frees the buffers.  You MUST
//!	do an MpFlush before closing the port.  
int MpClose (
	MPORT *port									//!< Port to close
	);

//! Determine if data available for reading.
//! 
//! @return TRUE if data available, FALSE if none, < 0 if error.
int MpComHasDataAvail (
	MPORT *port									
	);

//! Set input/output queue sizes for COM ports.
void MpComSetQueueSize (
	int InputSize, 
	int OutputSize
	);

//! Flush a buffered port.
//!
//! @return Port status.
//!
//!	This routine flushes the buffers for a buffered port.  If the
//!	returned status is not 0, the port has not been flushed, and
//!	you should flush it again.
int MpFlush (
	MPORT *port									//!< Port to flush
	);

//! Return last error that occurred for a port.
int MpGetLastError (
	MPORT *port									//!< Port, may be NULL
	);

//! Get a MPORTTYPE given a port type name.
//!
//! @return Pointer to the MPORTTYPE structure needed in MpOpen() or NULL if no such port type exists.
//!
//!	This routine takes the name of a port type and returns the corresponding MPORTTYPE structure.  
//! This name may be stored in tntproc.ini.  It is also returned by MpSelectPort.
MPORTTYPE *	MpGetPortTypeByName (
	const char *name							//!< Name of port type
	);

//! Get a MPORTTYPE given a port type number.
//!
//! @return Pointer to the MPORTTYPE structure needed in MpOpen() or NULL if no such port type exists.
//!
//! PORTTYPE_LPT		PORTTYPE_COM		PORTTYPE_FILE				
MPORTTYPE *	MpGetPortTypeByNumber (
	int num 										//!< Numeric port type
	);

//! Initialize the port library.
//!
//! This function initializes the MicroImages Port routines.
int MpInit (void);

//! Is file really a "file" ? 
//! 
//! @return 1 if a given file is really a "file", 0 if it's really a device (eg: LPT1: or /dev/tty0).
int MpIsFile (
	const char *type, 
	const char *name
	);

//! Is file really a "file" -- Unicode version ?
//!
//! @return 1 if a given file is really a "file", 0 if it's really a device (eg: LPT1: or /dev/tty0).
int MpIsFileUC (
	const char *type, 
	const MIUNICODE *name
	);

//! Return the last known status for a port.
//!
//! @return The last status value without querying the device.
int MpLastStatus (
	MPORT *port									
	);

//! Open a port.
//!
//! This routine will open a port.  To get the MPORTTYPE structure needed in the second parameter, 
//! use MpGetPortTypeByName or MpGetPortTypeByNumber.  The mode flags are listed in MpSetMode.
//!	The parms parameter is used to override the default parameters in tntproc.ini.  Normally, just pass 
//! NULL for this paramter.
DEPRECATED_MSG("There's a version that takes MISTRING for the name now") MPORT * MpOpen (
	const char *name, 						//!< Name of port/file to open
	const MPORTTYPE *type, 							//!< Type of port to use
	const char *parms = 0, 					//!< Parameters 
	UINT32 mode	= 0							//!< Mode flags
	);

//! Open a port.
//!
//! This routine will open a port.  To get the MPORTTYPE structure needed in the second parameter, 
//! use MpGetPortTypeByName or MpGetPortTypeByNumber.  The mode flags are listed in MpSetMode.
//!	The parms parameter is used to override the default parameters in tntproc.ini.  Normally, just pass 
//! NULL for this paramter.
MPORT * MpOpen (
	const MISTRING& name, 					//!< Name of port/file to open
	const MPORTTYPE *type, 					//!< Type of port to use
	const char *parms = 0, 					//!< Parameters 
	UINT32 mode	= 0							//!< Mode flags
	);

//! Read from a port.
//!
//! @return Bytes read >= 0; 0 if no data read.
//!
//!	WARNING:  This function does NOT return error codes, 0 will be returned if no data is read.
//!
//!	This function will read data from a given port.  If term is NULL, it acts like a normal read() 
//! function.  If term is not NULL, it should contain a NULL-terminated string containing terminators.
//!	For example, if term contains "\n\t ", MpRead will read up to the next newline, tab, or space, 
//! but not more than maxlen chars. The terminator itself will also be read.
//!
//! NOTE:  The "term" argument hasn't been implemented, and is currently ignored.
int MpRead (
	MPORT *port, 								//!< Port to read from
	void *buf, 									//!< Buffer to read into
	int maxlen, 								//!< Max to read
	char *term = 0									//!< List of terminators
	);


#ifndef GENERATING_DOXYGEN_OUTPUT 
int MpScan (
	MPORTTYPE *porttype
	);
#endif //!< GENERATING_DOXYGEN_OUTPUT


//! Set port specific parameters.
//!
//! This routine allows you to set parameters for a given port. The values of the parms parameter is 
//! dependant on the port, but should always be in the following format:		
//!
//! 		:parm1:parm2:parm3:
//!
//! This string may be stored in tntproc.ini.  The last user selected parameters for each port are 
//! stored in tntproc.ini and automatically used when you call MpOpen.  This function is only here to 
//! allow you to override them.	
//! 
//! Example:		
//!			:9600:8:none:1:
//!	for a "serial_port", this means 9600 baud, 8 data bits, no parity, and 1 stop bit.
int MpSetParms (
	MPORT *port, 								//!< Port to perform operation on
	const char *parms							//!< Parameters to set
	);

//! Get the status of a port.
//!
//! @return Status code.
//!
//! This function will query the port and return its status.  Some types of ports return 
//! detailed status information. The status codes shown below are bits, so it's possible
//! to get more than one at a time.  If the status string is not wanted, pass NULL for status.
//! 
//!	Possible status codes returned:
//!		0							OK		
//!		PORTSTAT_Error			Some error has occured
//!		PORTSTAT_OffLine		The printer is off-line
//!		PORTSTAT_OutOfPaper	The printer is out of paper
//!		PORTSTAT_NotReady		The printer is busy		
//!		PORTSTAT_Abort
//!
int MpStatus (
	MPORT *port, 								//!< Port to perform operation on
	char **status								//!< Pointer to status RETURNED
	);

//! Write to a port.
//!
//! @return Bytes written >= 0; 0 if no data written.
//!
//!	WARNING:  This function does NOT return error codes, 0 will be returned if no data is written.
//!
//!	This function will write data to the given port.  If the port is buffered, it might not actually 
//! send unless the buffer fills up or until you call MpFlush.  If the status returned is not 0,
//!	the port is busy and the write has been buffered.  In this case, you should call MpFlush() until 
//! it returns a status of 0 or until the user gets bored and goes home. If len is -1, buf is assumed 
//! to be a NULL-terminated string.
//!
//! If the PORTMODE_SingleStream flag is set, the data is assumed to be wrapped as follows...			
//!
//! 		\<CC\> \<HH\> \<LL\> \<Data....\>
//! 
//!	Where CC is the channel number (0 == CDB, 1 == DATA) to write to, HH and LL are the high and 
//! low-bytes of the length to write.  In this mode, data	is buffered until the full length is 
//! recieved and is then sent all at once.  This is useful for where data has to be 
//! sent in fixed size chunks.  A data block can span multiple calls to MpWrite().  MpWrite() will 
//! write at most one block	per call.  If a buffer contains multiple blocks, MpWrite() will
//!	write the first block then return a count indicating how many bytes were written.  It is up to 
//! the caller to call MpWrite	again with the rest of the buffer (possibly checking MpStatus()
//!	between calls).  Also, to send just a CDB (eg reset command) using this method, you have to 
//! send the CDB in channel 0, followed by a 0-byte data block on channel 1.
int MpWrite (
	MPORT *port, 								//!< Port to write to
	const void *buf, 							//!< Buffer to send
	int len										//!< Length of buffer
	);

//! Same as MpWrite() but if the port becomes busy, will add a message to the current status handle 
//! (if any) and check the cancle button.
int MpWriteExt (
	MPORT *port, 
	const void *buffer, 
	int len, 
	UINT32 flags = 0
	);

//! Write to a port with status and/or error dialogs. 
//!
//! @return 0 or EUserCancel.
//!
//! This function is like MpWrite(), but it will keep trying if the port is
//!	busy.  If there is an error on the port, the error is displayed in the status dialog
//!	in the specified line.  If line is == -1, then "status" is used as the parent widget
//!	for a Retry/Cancel dialog instead of as a status dialog.
int MxpWrite (
	MPORT *port, 								//!< Port to write to (see MpOpen)
	const void *buf, 							//!< Buffer of stuff to write
	int numbytes, 								//!< Number of bytes to write
	void *status, 								//!< Status dialog
	int line										//!< Line in status dialog to put errors in
	);

#ifndef PORTS_C
extern MPORTTYPE *MpPortTypeList[];
#endif

//! Create dialog for port settings.
int MxPortGetSettings (
	Widget pwidget, 
	MPORTTYPE *porttype, 
	char *portoptions
	);


#endif	//!<  INC_MI32_PORTS_H 
