/**
 * \file xmipane.h <mi32/xmipane.h>
 * \brief Definitions for XmiPane widget.
 *
 * \if NODOC
 * $Id: xmipane.h_v 1.14 2010/10/15 19:52:19 mju Exp $
 *
 * $Log: xmipane.h_v $
 * Revision 1.14  2010/10/15 19:52:19  mju
 * Add xmiPANE_SIZEALIGN_Ignore.
 *
 * Revision 1.13  2010/02/26 17:54:20  mju
 * Add sizealign_ExcludeFromLayout.
 *
 * Revision 1.12  2008/09/03 13:36:04  mju
 * Fix doxygen warnings.
 *
 * Revision 1.11  2007/12/27 21:14:52  mju
 * Add proportional resource setting.
 *
 * Revision 1.10  2005/11/30 15:25:06  mju
 * REmove computeSize as does not work.
 *
 * Revision 1.9  2005/11/30 14:46:53  mju
 * Add xmiPaneComputeSize
 *
 * Revision 1.8  2004/03/08 15:37:15  mju
 * Fix comment for old mgui hdr.
 *
 * Revision 1.7  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.6  2002/02/20 16:01:29  mju
 * Add comments for support XtNminHeight/Width constraint resources.
 *
 * Revision 1.5  2001/12/20 14:37:55  mju
 * Change comment about spacing to say not added around border.
 *
 * Revision 1.4  2001/12/11 17:54:50  mju
 * Add orientations for horizontal LtoR and RtoL.
 *
 * Revision 1.3  2001/12/11 17:17:23  mju
 * Change default sizealign to FixedSize.
 *
 * Revision 1.2  2001/12/11 16:59:36  mju
 * Add documentation comments.
 *
 * Revision 1.1  2001/12/10 21:17:03  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_XMIPANE_H
#define	INC_MI32_XMIPANE_H

#ifndef _XmBulletinBoard_h
#include <Xm/BulletinB.h>
#endif

extern "C" {

//!:Associate with "XmiPane widget functions"
//!\addtogroup XmiPane XmiPane Widget Functions
//!@{

extern WidgetClass xmiPaneWidgetClass;

//! Add expanding space to pane.
//! Space will be inserted after the most recently added child, or
//! if no children have yet been added, before the first child.
void XmiPaneAddSpaceExpanding (
	Widget PaneWidget
	);

//! Add fixed space to pane.
//! Space will be inserted after the most recently added child, or
//! if no children have yet been added, before the first child.
void XmiPaneAddSpaceFixed (
	Widget PaneWidget,
	int size
	);

//! Create unmanaged XmiPane widget.
//!
//! An XmiPane is a composite container widget which places its children in either a
//! horizontal or vertical "pane".  Children are arranged in the order in which they
//! are added, with optional fixed and expanding space between each child and before
//! the first child.
//!
//! XmiPane inherits behavior from XmBulletinBoard.
//! The class pointer is xmiPaneWidgetClass.
//!
//!
//! The following resources are available for XmiPane:
//!		Name							Default									Access
//!			Class							Type
//!		XmNmarginHeight			0											CSG
//!			XmCMarginHeight			Dimension							(Inherited from BulletinBoard)
//!		XmNmarginWidth				0											CSG
//!			XmCMarginWidth				Dimension							(Inherited from BulletinBoard)
//!		XtNorientation				XmiPANE_ORIENTATION_HORIZONTAL	CG
//!			XtCOrientation				unsigned char
//!		XmiNspacing					4											CSG
//!			XmiCSpacing					Dimension
//!		XmiNignoreNonVisible		False										CSG
//!			XmiCIgnoreNonVisible		Boolean
//!		XmiNproportional			False										CSG
//!			XmiCProportional			Boolean
//!
//!	XtNorientation
//!		Determines whether children are horizontally or vertically.  Possible values are:
//!			XmiPANE_ORIENTATION_HORIZONTAL			Order horizontally based on locale
//!			XmiPANE_ORIENTATION_HORIZONTAL_LTOR		Order horizontally left to right
//!			XmiPANE_ORIENTATION_HORIZONTAL_RTOL		Order horizontally right to left
//!			XmiPANE_ORIENTATION_VERTICAL				Order vertically top to bottom
//!		Values are equal to the corresponding MGUI::LAYOUT_ORIENTATION enum values in <mgui/layout.h>.
//!
//!	XmiNspacing
//!		Determines spacing between children in the pane orientation.  This spacing is not added
//!		between the children and pane edges, use XmNmarginWidth and XmNmarginHeight in that case.
//!
//!	XmiNignoreNonVisible
//!		Determines whether space for nonvisible (unmanaged) children is retained in the layout
//!		or not.  The default of False will retain space, thus retaining all managed child
//!		positions.  Setting this resource to True will cause the Pane to adjust the positions
//!		of remaining visible children to fill the space left by the unmanaged child(ren).
//!
//!	XmiNproportional
//!		Determines whether children are resized proportional to previous size.
//!		The default of False will add space to all resizable children equal.  Setting this
//!		resource to True will maintain current proportions of resizable children, subject to
//!		minimum size constraints.  This is primarily used when a sash control is added to the pane.
//!
//!
//!	The following constraint resources are available to children of XmiPane:
//!		Name							Default									Access
//!			Class							Type
//!		XmiNsizeAlign				XmiPANE_SIZEALIGN_FixedSize		CG
//!			XmiCSizeAlign				unsigned short
//!		XtNminHeight				[none]									CSG
//!			XtCMinHeight				Dimension
//!		XtNminWidth					[none]									CSG
//!			XtCMinWidth					Dimension
//!
//!	XmiNsizeAlign
//!		Determines how a child is sized and aligned with other children in the parent XmiPane.
//!		The values for this resource may be bitwise-or'd (|) together.  In addition, these values
//!		exactly correspond to the MGUI::LAYOUT_SIZEALIGN enum values defined in <mgui/layout.h>
//!
//!		XmiPANE_SIZEALIGN_Expand				Expand both width and height to fill space
//!		XmiPANE_SIZEALIGN_FixedWidth			Width fixed, height can change
//!		XmiPANE_SIZEALIGN_FixedHeight			Height fixed, width can change
//!		XmiPANE_SIZEALIGN_FixedSize			Both horizontal and vertical size fixed
//!		XmiPANE_SIZEALIGN_AlignLeft			Align with left edge of available space
//!		XmiPANE_SIZEALIGN_AlignRight			Align with right edge of available space
//!		XmiPANE_SIZEALIGN_AlignTop				Align with top edge of available space
//!		XmiPANE_SIZEALIGN_AlignBottom			Align with bottom edge of available space
//!		XmiPANE_SIZEALIGN_CenterHorz			Center horizontally in available space
//!		XmiPANE_SIZEALIGN_CenterVert			Center vertically in available space
//!		XmiPANE_SIZEALIGN_Center				Center both horizontal and vertically
//!
//!	XtNminHeight
//!		Minimum height of child in pixels.  If not specified will default to the initial child height.
//!
//!	XtNminWidth
//!		Minimum width of child in pixels.  If not specified will default to the initial child width.
//!		
Widget XmiPaneCreate (
	Widget parent,
	char *name,
	ArgList arglist,
	Cardinal argcount
	);

//! Create Dialog and XmiPane widget.
Widget XmiPaneCreateDialog (
	Widget parent,
	char *name,
	ArgList arglist,
	Cardinal argcount
	);

//! Determine if widget is an XmiPane or subclass thereof.
inline Boolean XmiIsPane (
	Widget w
	) {
	return (XtIsSubclass(w, xmiPaneWidgetClass));
	}

//! Pane child placement orientation.
// These values must remain equal to the corresponsing MGUI::LAYOUT_ORIENTATION values in <mgui/layout.h>
#define	XmiPANE_ORIENTATION_HORIZONTAL		0				//!< Horizontal layout based on locale
#define	XmiPANE_ORIENTATION_VERTICAL			1				//!< Vertical layout top to bottom
#define	XmiPANE_ORIENTATION_HORIZONTAL_LTOR	2				//!< Horizontal layout left to right
#define	XmiPANE_ORIENTATION_HORIZONTAL_RTOL	3				//!< Horizontal layout right to left

//! Child sizing/alignment constraint values, can be |'d together.
// These values must remain equal to the corresponding MGUI::LAYOUT_SIZEALIGN values in <mgui/layout.h>
#define	XmiPANE_SIZEALIGN_Expand				0x0000		//!< Expand both width and height to fill space
#define	XmiPANE_SIZEALIGN_FixedWidth			0x0001		//!< Width fixed, height can change
//#define	XmiPANE_SIZEALIGN_RelativeHorz		0x0002
#define	XmiPANE_SIZEALIGN_FixedHeight			0x0004		//!< Height fixed, width can change
//#define	XmiPANE_SIZEALIGN_RelativeVert		0x0008
#define	XmiPANE_SIZEALIGN_FixedSize			0x0005		//!< Both horizontal and vertical size fixed
#define	XmiPANE_SIZEALIGN_AlignLeft			0x0010		//!< Align with left edge of available space
#define	XmiPANE_SIZEALIGN_AlignRight			0x0020		//!< Align with right edge of available space
#define	XmiPANE_SIZEALIGN_AlignTop				0x0040		//!< Align with top edge of available space
#define	XmiPANE_SIZEALIGN_AlignBottom			0x0080		//!< Align with bottom edge of available space
#define	XmiPANE_SIZEALIGN_CenterHorz			0x0030		//!< Center horizontally in available space
#define	XmiPANE_SIZEALIGN_CenterVert			0x00C0		//!< Center vertically in available space
#define	XmiPANE_SIZEALIGN_Center				0x00F0		//!< Center both horizontal and vertically
#define	XmiPANE_SIZEALIGN_ExcludeFromLayout	0x0100		//!< Exclude from layout, will be placed at top/left unless other Align/Center values are set
#define	XmiPANE_SIZEALIGN_Ignore				0x0200		//!< Ignore completely if exclude from layout, will never be resized or moved by pane


#define	XmiNspacing					"spacing"
#define	XmiCSpacing					"Spacing"
#define	XmiNignoreNonVisible		"ignoreNonVisible"
#define	XmiCIgnoreNonVisible		"IgnoreNonVisible"
#define	XmiNsizeAlign				"sizeAlign"
#define	XmiCSizeAlign				"SizeAlign"
#define	XmiNproportional			"proportional"
#define	XmiCProportional			"Proportional"

//!@}

}  /* Close scope of 'extern "C"' declaration which encloses file. */

#endif	//!< INC_MI32_XMIPANE_H
