/*
 * \if NODOC
 *
 *
 *
 *
 *
 * DO NOT ADD MORE FUNCTIONS OR CLASSES HERE.  Use the RVC::OP namespace and a header specific to the class being added.
 *
 *
 *
 *
 *
 *
 *	$Log: rasterop.h_v $
 *	Revision 1.19  2011/09/06 13:54:13  mju
 *	Exclude macro from dox.
 *
 *	Revision 1.18  2007/04/04 20:23:59  bcolombe
 *	Added mask objitem parameter to CombineLinear
 *	Removed InvertMask bool parameter from ComputeMultLinRegression
 *
 *	Revision 1.17  2007/03/09 15:40:29  drb
 *	Build warnings
 *
 *	Revision 1.16  2006/11/28 17:45:30  mju
 *	Remove computeSAS and related enums as no longer used.
 *
 *	Revision 1.15  2006/11/10 17:47:15  dfriberg
 *	Add more methods
 *
 *	Revision 1.14  2006/09/21 13:05:42  mju
 *	Use inclusion guards, and only include headers needed to compile in isolation.
 *
 *	Revision 1.13  2006/08/07 13:57:51  mju
 *	Add comment to not add more stuff here.
 *
 *	Revision 1.12  2006/08/03 20:16:30  dwilliss
 *	added Clump and Sieve functions
 *
 *	Revision 1.11  2006/06/29 01:11:52  dfriberg
 *	Allow settings compression options in CombineLinear
 *
 *	Revision 1.10  2006/06/28 21:26:15  dfriberg
 *	CombineLinear uses raster pipeline
 *
 *	Revision 1.9  2005/08/18 19:17:27  dfriberg
 *	Changed MakeSurfaceProperties to take const RVC::OBJITEM&
 *
 *	Revision 1.8  2005/08/18 19:15:10  lt
 *	Cleaned up
 *
 *	Revision 1.7  2005/08/17 22:48:22  dfriberg
 *	Change to namespace from class, added MakeSurfaceProperties method
 *
 *	Revision 1.6  2005/08/12 17:32:05  lt
 *	Removed unnecessary method from the rasterop class
 *
 *	Revision 1.5  2005/08/08 16:05:30  dfriberg
 *	Documented CombineLinear
 *
 *	Revision 1.2  2005/07/11 20:36:04  lt
 *	Added CompSAS
 *
 *	Revision 1.1  2004/12/22 19:44:12  dfriberg
 *	Initial revision
 *
 * \endif
 */

#ifndef INC_RVC_RASTEROP_H
#define INC_RVC_RASTEROP_H

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
class MIMATRIX;
struct PRINCOMPPARMS;
#endif

namespace RVC {

	//!	\namespace RVC::RASTEROP rvc/rasterop.h
	//!	\brief Contains static methods for raster operations
	namespace RASTEROP {

		enum CLUMPFLAGS {
			CLUMPFLAG_Larger = 0x00000000,		//<! Find clumps >= given size 
			CLUMPFLAG_Smaller = 0x00000001,		//<! Find clumps <= given size 
			CLUMPFLAG_FillHoles = 0x00000002,	//<! Fill holes in clumps
			};
#ifndef GENERATING_DOXYGEN_OUTPUT
		DEFINE_ENUM_OP_BITWISE(CLUMPFLAGS)
#endif

		//! Compute a "Clump" raster from a given input raster.
		ERRVALUE ComputeClumpRaster (
			const RVC::OBJITEM& InRastObjItem,
			RVC::OBJITEM& OutRastObjItem,
			UINT32 ClumpSize,
			CLUMPFLAGS flags = CLUMPFLAG_Larger
			);

		//! Combine input rasters using linear transformation stored in Matrix.
		//! Creates \f$m\f$ output rasters for \f$n\f$ input rasters.  Each output raster cell of raster
		//! \f$i\f$ (for \f$1 \leq i \leq m\f$) is computed with
		//! \f$out_i = Matrix_{i1} * in_1 + Matrix_{i2} * in_2 + ... + Matrix_{in} * in_n + Offsets_i\f$,
		//! where Matrix is stored in row major form.
		//!
		//! Destination rasters will be created based on the contents of
		//! DestRastList.  Number of Matrix rows and size of Offsets must
		//! equal the number of destination rasters (i.e. size of DestRastList),
		//! and the number of Matrix columns must equal the number of source
		//! rasters.
	 	ERRVALUE CombineLinear (
			const RVC::OBJITEMLIST& SrcRastList,					//!< Input raster objects
			RVC::OBJITEMLIST& DestRastList,							//!< Output raster objects
			RVC::IMAGE::CELLTYPE celltype,							//!< Output rasters' cell type
			const MIMATRIX& Matrix,										//!< Linear transformation Matrix, row major
			const MIMATRIX& Offsets,									//!< Array of Offsets
			bool AutoScale,												//!< Auto scale output raster values.
			const RVC::OBJITEM* MaskObj = NULL,
			RVC::RASTER::COMPTYPE compression=RVC::RASTER::COMPTYPE_None,
			UINT16 compquality=0
			);

		ERRVALUE ComputeMultLinRegression (
			MIMATRIX& coefficients,
			MIMATRIX& offsets,
			const RVC::OBJITEMLIST& ObjItems,
			const RVC::OBJITEM& MaskObj,
			INT32 colsample,						/* sampling rates for analysis */
			INT32 linsample
			);

		ERRVALUE ComputePrinComp (
			MIMATRIX& forward,
			MIMATRIX& inverse,
			PRINCOMPPARMS& parm,								/* PRINCOMPPARMS *parm */
			const RVC::OBJITEMLIST& InRasters,
			const RVC::OBJITEM* MaskObj = NULL,
			UINT32 colsample = 1,						/* sampling rates for analysis */
			UINT32 linsample = 1,
			UINT32 flags = 0
			);


		//! Compute a "Sieve" raster from a given input raster.
		ERRVALUE ComputeSieveRaster (
			const RVC::OBJITEM& InRastObjItem,
			RVC::OBJITEM& OutRastObjItem,
			UINT32 ClumpSize,
			CLUMPFLAGS flags = CLUMPFLAG_Larger
			);

		//! Compute 3d surface properties and store as subobject of
		//! raster referenced by objitem.
		ERRVALUE MakeSurfaceProperties (
			const RVC::OBJITEM& objitem
			);
		}
	}
#endif
