/**
 * \file rvc/vector.h 
 * \brief interface for the RVC::VECTOR class.
 *
 * \if NODOC
 * $Id: vector.h_v 1.132 2012/10/09 15:01:01 mju Exp $
 *
 * $Log: vector.h_v $
 * Revision 1.132  2012/10/09 15:01:01  mju
 * Add "separate" to setupElemTransList.
 *
 * Revision 1.131  2011/10/10 13:50:24  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.130  2011/09/02 13:20:56  mju
 * Exclude macros from dox.
 *
 * Revision 1.129  2010/04/21 14:28:03  mju
 * Change elementBegin/End to take const reference to avoid copy ctor.
 *
 * Revision 1.128  2009/10/28 21:47:58  mju
 * Make ELEMITERATOR ctor more efficent for End and don't need virtual dtor.
 *
 * Revision 1.127  2008/09/09 21:58:07  scowan
 * Added methods for extract to shape object.
 *
 * Revision 1.126  2007/09/20 19:15:56  scowan
 * Removed legacy vector element structure names.
 *
 * Revision 1.125  2007/06/29 23:21:20  scowan
 * Converted all remaining vector legacy structs to children of vector class.
 *
 * Revision 1.124  2007/04/12 21:49:27  scowan
 * Support rvc vector level extents caching system.
 *
 * Revision 1.123  2007/04/04 20:05:29  scowan
 * Added addRegion and addSubRegion methods.
 *
 * Revision 1.122  2007/02/16 23:39:33  scowan
 * Added needed include file.
 *
 * Revision 1.121  2006/12/29 21:15:50  scowan
 * Added readElement virtual methods.
 *
 * Revision 1.120  2006/07/20 20:24:30  mju
 * Add optimizeTo using objitem.
 *
 * Revision 1.119  2006/06/27 15:05:17  scowan
 * Make method now fills in objitem with created object info.
 *
 * Revision 1.118  2006/02/07 14:53:25  scowan
 * Added methods to set via rvc::element.
 *
 * Revision 1.117  2006/02/01 18:27:11  scowan
 * Added generate nodes method.
 *
 * Revision 1.116  2005/12/21 23:08:59  scowan
 * Added support for the readNext virtual methods.
 *
 * Revision 1.115  2005/12/20 18:23:19  dwilliss
 * Adjusted mandatory credentials to account for prior optional parameters
 *
 * Revision 1.114  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.113  2005/12/09 17:30:04  scowan
 * Updated to use credentials.
 *
 * Revision 1.112  2005/10/20 17:01:30  scowan
 * Quadtree and geometric interface changes supported.
 *
 * Revision 1.111  2005/07/15 17:25:59  scowan
 * Added addPoint method.
 *
 * Revision 1.110  2005/07/08 16:18:53  scowan
 * Removed deprecated make method.
 *
 * Revision 1.109  2005/02/23 18:13:43  scowan
 * Removed unused legacy method.
 *
 * Revision 1.108  2005/01/19 21:12:02  scowan
 * Added support for descriptor.
 *
 * Revision 1.107  2004/12/18 00:08:02  scowan
 *  Added component field to findClosestElements.
 *
 * Revision 1.106  2004/11/10 21:37:23  scowan
 * Added threshold parms to is equiv methods.
 *
 * Revision 1.105  2004/10/27 21:15:41  scowan
 * Added new private method to read label baseline.
 *
 * Revision 1.104  2004/09/08 17:06:50  scowan
 * No real change.
 *
 * Revision 1.103  2004/08/24 22:26:27  scowan
 * Added virtual methods to get and set total line length.
 *
 * Revision 1.102  2004/07/19 17:03:22  scowan
 *  Moved has min elem filter to geometric class.
 *
 * Revision 1.101  2004/07/14 21:13:40  scowan
 * Removed legacy forward declaration.
 *
 * Revision 1.100  2004/07/12 23:21:44  scowan
 * Removed use of geo region.
 *
 * Revision 1.99  2004/06/01 22:25:01  scowan
 * Added cad and region extract to methods.
 *
 * Revision 1.98  2004/05/28 20:35:48  scowan
 * Added new virtual method override.
 *
 * Revision 1.97  2004/03/20 18:24:13  scowan
 *  Added find closest elements override.
 *
 * Revision 1.96  2004/03/17 21:31:34  scowan
 * Added new label get extents method.
 *
 * Revision 1.95  2004/03/01 17:49:31  scowan
 * Added docs.
 *
 * Revision 1.94  2004/02/23 20:06:48  scowan
 * Make get obj filter derived class inherit publically.
 *
 * Revision 1.93  2004/02/10 23:12:57  scowan
 * Added makeparm ctor.
 *
 * Revision 1.92  2004/02/10 17:00:02  scowan
 * Added two new add methods.
 *
 * Revision 1.91  2004/02/06 21:54:34  scowan
 * Fixed warning.
 *
 * Revision 1.90  2004/01/08 22:53:30  scowan
 * Fixed op enum issue.
 *
 * Revision 1.89  2003/12/04 21:18:08  scowan
 * Added compare helper methods.
 *
 * Revision 1.88  2003/11/25 17:14:14  scowan
 * Added method to equate two vector objects..
 *
 * Revision 1.87  2003/11/21 22:17:12  scowan
 * Support new Capabilities system.
 *
 * Revision 1.86  2003/10/21 21:59:30  scowan
 * Added more filters.
 *
 * Revision 1.85  2003/10/10 23:01:05  scowan
 * Added filter.
 *
 * Revision 1.84  2003/10/07 19:43:43  scowan
 * nc.
 *
 * Revision 1.83  2003/10/07 17:30:04  scowan
 * Object virtual method changes.
 *
 * Revision 1.82  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.81  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.80  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.79  2003/08/27 21:26:25  scowan
 * Uses new rvc vector class interface and handle.
 *
 * Revision 1.78  2003/06/19 17:46:22  scowan
 * Added virtual method.
 *
 * Revision 1.77  2003/06/04 23:35:19  scowan
 * Added find element draw set.
 *
 * Revision 1.76  2003/05/12 23:33:25  scowan
 * Changes to virtual methods.
 *
 * Revision 1.75  2003/05/07 22:20:55  scowan
 * Changed virtual method format.
 *
 * Revision 1.74  2003/03/11 21:38:17  scowan
 * Changed label get extents to use 3d rect.
 *
 * Revision 1.73  2003/03/04 21:04:44  scowan
 * Changed to default to no element ID tables.
 *
 * Revision 1.72  2003/02/19 20:14:05  scowan
 * Something, I do not know what.
 *
 * Revision 1.71  2003/02/11 00:06:27  scowan
 * More const.
 *
 * Revision 1.70  2003/01/16 23:00:53  scowan
 * Added bool value to the class.
 *
 * Revision 1.69  2002/12/23 20:30:28  scowan
 * *** empty log message ***
 *
 * Revision 1.68  2002/11/07 16:58:32  scowan
 * Added optimize to method.
 *
 * Revision 1.67  2002/10/29 16:57:25  scowan
 * Changed method name.
 *
 * Revision 1.66  2002/10/23 14:56:56  scowan
 * Re-enable subregion and region reading code.
 *
 * Revision 1.65  2002/10/18 14:29:02  scowan
 * Comment out region code until I get it working again.
 *
 * Revision 1.64  2002/10/18 13:48:04  scowan
 * Damn mac compiler..
 *
 * Revision 1.63  2002/10/11 16:59:50  scowan
 * Uses bitset unwoned for read region.
 *
 * Revision 1.62  2002/10/11 16:45:12  scowan
 * Added read subregion and region.
 *
 * Revision 1.61  2002/10/07 23:23:27  scowan
 * Added another field.
 *
 * Revision 1.60  2002/10/07 17:39:55  scowan
 * More comment changes.
 *
 * Revision 1.59  2002/10/07 15:55:25  scowan
 * Fixed docs.
 *
 * Revision 1.58  2002/10/04 21:18:29  scowan
 * Added elem iterator decrement operator.
 * Added protected methods.
 *
 * Revision 1.57  2002/10/01 22:43:44  scowan
 * Another protected metod.
 *
 * Revision 1.56  2002/10/01 21:37:01  scowan
 * Added protected method.
 *
 * Revision 1.55  2002/09/30 23:22:12  scowan
 * Made element search code const.
 *
 * Revision 1.54  2002/09/18 15:41:05  scowan
 * Added clip polygon method.
 *
 * Revision 1.1  2000/10/02 17:40:07  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_VECTOR_H
#define INC_RVC_VECTOR_H

#ifndef INC_RVC_GEOMETRC_H
	#include <rvc/geometrc.h>
#endif

#ifndef INC_MI32_ERRHANDLER_H
#include <mi32/errhandler.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
class BITSET_UNOWNED;
class TRANS2D_MAPGEN;
namespace RVC {
	class CREDENTIALS;
	class DBASE_ELEMENT;
	class DBTABLE;
	class SHAPE;
	class SPATIALTREE;
	class STYLE;
	class VECTORHANDLE;
	}
#endif

namespace RVC {

//!	Manages RVC::OBJTYPE_Vector RVC objects.  
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Vector.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Vector.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
class VECTOR : public GEOMETRIC {
	public:

		enum FLAGS {
			FLAG_None = 0,
			FLAG_NoStdAttribTables = 0x0001,
			FLAG_NoElementIDTables = 0x0002,
			FLAG_CannotReduceExtents = 0x0004,
			FLAG_CannotExpandExtents = 0x0008
			};

		enum TOPOLOGYTYPE {
			TOPOLOGYTYPE_Polygonal = 0,		//!< Vector object has polygons and full polygon-line-node topology
			TOPOLOGYTYPE_Planar, 			//!< Vector object has no polygons and full line-node topology
			TOPOLOGYTYPE_Network,			//!< Vector object has no polygons and connected line-node topology
			TOPOLOGYTYPE_None				//!< Vector object has no topology, lines only
			};

		enum SUBTYPE {
			SUBTYPE_2DPolygonal = 0,
			SUBTYPE_2DPlanar,
			SUBTYPE_2DNetwork,
			SUBTYPE_3DPolygonal,
			SUBTYPE_3DPlanar,
			SUBTYPE_3DNetwork,
			SUBTYPE_3DNone
			};

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class CONGLOMERATELINEFILTER;
		class GETOBJFILTER_HASGEOMETRICELEMENTS;
		class GETOBJFILTER_HASLABELS;
		class GETOBJFILTER_HASLINES;
		class GETOBJFILTER_HASPOINTS;
		class GETOBJFILTER_HASPOLYGONS;
		class LABEL;
		struct LINE;
		class MAKEPARMS;
		struct NODE;
		struct POINT;
		struct POLYGON;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Element iterator, used to iterate through valid vector object elements.
		//! See methods ElementBegin() and ElementEnd() for obtaining an element iterator
		template <typename _ELEM> class ELEMITERATOR {
			public:

				//! Default constructor
				ELEMITERATOR (
					) : 
					m_ElemNum(-1),
					m_pObjVector(0),
					m_pElement(0)
					{}

				//! Idiot constructor for G++
				ELEMITERATOR (
					int
					) : 
					m_ElemNum(-1),
					m_pObjVector(0),
					m_pElement(0)
					{}

				//! Copy constructor
				ELEMITERATOR (
					const ELEMITERATOR<_ELEM>& rhs
					) : 
					m_ElemNum(rhs.m_ElemNum),
					m_pObjVector(rhs.m_pObjVector),
					m_pElement((rhs.m_pElement != 0) ? new _ELEM(*rhs.m_pElement) : 0)
					{}

				//! Destructor
				~ELEMITERATOR (
					) {
					delete m_pElement;
					}

				//! Assignment
				ELEMITERATOR<_ELEM>& operator= (
					const ELEMITERATOR<_ELEM>& rhs
					) {
					if (this != &rhs) {
						m_ElemNum = rhs.m_ElemNum;
						m_pObjVector = rhs.m_pObjVector;
						if (rhs.m_pElement != 0) {
							if (m_pElement != 0) {
								*m_pElement = *rhs.m_pElement;
								}
							else {
								m_pElement = new _ELEM(*rhs.m_pElement);
								}
							}
						else {
							delete m_pElement;
							m_pElement = 0;
							}
						}
					return (*this);
					}

				//! Dereference
				//! Do not use unless iterator obtained from VECTOR::ElementBegin() or will crash.
				const _ELEM& operator* (
					) const { return (*m_pElement); }

				//! Dereference arrow
				const _ELEM* operator-> (
					) const { return (&**this); }

				//! Pre-increment to next valid element.
				//! Do not use unless iterator obtained from VECTOR::ElementBegin() or will crash.
				ELEMITERATOR<_ELEM>& operator++ (
					) {
					ERRORPOSNDISABLE disable;
					do {
						++m_ElemNum;
						if (m_pObjVector->Read(m_ElemNum, *m_pElement) < 0) {
							m_ElemNum = -1;
							break;
							}
						} while (!m_pElement->IsValid());
					return (*this);
					}			

				//! Pre-decrement to previous valid element.
				//! Do not use unless iterator obtained from VECTOR::ElementBegin() or will crash.
				ELEMITERATOR<_ELEM>& operator-- (
					) {
					if (m_ElemNum < 0) return (*this);
					ERRORPOSNDISABLE disable;
					do {
						--m_ElemNum;
						if (m_pObjVector->Read(m_ElemNum, *m_pElement) < 0) {
							m_ElemNum = -1;
							break;
							}
						} while (!m_pElement->IsValid());
					return (*this);
					}			

				//! Comparison for equality
				bool operator== (
					const ELEMITERATOR<_ELEM>& rhs
					) const { return (m_ElemNum == rhs.m_ElemNum && m_pObjVector == rhs.m_pObjVector); }

				//! Comparison for inequality
				bool operator!= (
					const ELEMITERATOR<_ELEM>& rhs
					) const { return (!(*this == rhs)); }

				//! Get absolute element number.					
				INT32 GetElementNumber (
					) const { return (m_ElemNum); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				friend class RVC::VECTOR;

				INT32 m_ElemNum;
				const VECTOR *m_pObjVector;
				_ELEM *m_pElement;

				//! Internal constructor used by VECTOR::ElementBegin().
				explicit ELEMITERATOR (
					const RVC::VECTOR *pObjVector
					) :
					m_ElemNum(-1),
					m_pObjVector(pObjVector),
					m_pElement(new _ELEM)
					{
					operator++();	// Increment to first valid element
					}

				//! Internal constructor used by VECTOR::ElementEnd().
				ELEMITERATOR (
					const RVC::VECTOR *pObjVector,
					INT32 ElemNum
					) : 
					m_ElemNum(ElemNum),
					m_pObjVector(pObjVector),
					m_pElement(0)
					{}

				#endif
			};

		typedef ELEMITERATOR<LABEL> LABELITERATOR;
		typedef ELEMITERATOR<LINE> LINEITERATOR;
		typedef ELEMITERATOR<NODE> NODEITERATOR;
		typedef ELEMITERATOR<POINT> POINTITERATOR;
		typedef ELEMITERATOR<POLYGON> POLYGONITERATOR;
		
		//! Default constructor
		VECTOR (
			);

		//! Copy constructor
		VECTOR (
			const RVC::VECTOR& rhs
			);

		//! Destructor
		virtual ~VECTOR (
			);

		//! Assignment, will close and re-open object if possible
		RVC::VECTOR& operator= (
			const RVC::VECTOR& rhs
			);

		//! Add a label to the vector object
		//! @return Number of the added label element or < 0 if error
		INT32 Add (
			LABEL& label
			);

		//! Add a line to the vector object
		//! @return Number of the added line element or < 0 if error
		INT32 Add (
			const LINE& line
			);

		//! Add a node to the vector object
		//! @return Number of the added node element or < 0 if error
		INT32 Add (
			const NODE& node
			);

		//! Add a point to the vector object
		//! @return Number of the added point element or < 0 if error
		INT32 Add (
			const POINT& point
			);

		//! Add a polygon to the vector object
		//! @return Number of the added polygon element or < 0 if error
		INT32 Add (
			const POLYGON& polygon
			);
			
		//! Add a line to the vector object from a POLYLINE
		//! Will correctly set the LINE and update the extents of the vector
		//! Will not add nodes nor correct topology
		//! @return Number of the added line element or < 0 if error
		INT32 AddLine (
			const POLYLINE& PolyLine
			);
			
		//! Add a polygon to the vector object
		//! Will correctly set the POLYGON and update the extents of the vector
		//! Will add the line and set LINE correctly
		//! Will not add nodes nor correct topology
		//! 'PolyLine' element MUST be closed, or an error is returned
		//! @return Number of the added polygon element or < 0 if error
		INT32 AddPolygon (
			const POLYLINE& Polygon
			);
			
		//! Add a point to the vector object from a DPOINT3D
		//! Will correctly update the extents of the vector
		//! Will not correct topology
		//! @return Number of the added point element or < 0 if error
		INT32 AddPoint (
			const DPOINT3D& Point
			);
			
		//! Add a region to the vector object
		//! Will correctly set the POLYGON and update the extents of the vector
		//! Will correctly add the islands and set the parent polygon island set.
		//! Will add the lines and set LINE correctly
		//! Will not add nodes nor correct topology. 
		//! Assumes that the 'Region' is in the same Coordinate Reference System (CRS) as the vector object.
		//! Since multiple polygon shells can be added, the added element number cannot be returned
		ERRVALUE AddRegion (
			const REGION2D& Region
			);
			
		//! Add a sub-region to the vector object
		//! Will correctly set the POLYGON and update the extents of the vector
		//! Will correctly add the islands and set the parent polygon island set.
		//! Will add the lines and set LINE correctly
		//! Will not add nodes nor correct topology
		//! Assumes that the 'SubRegion' is in the same Coordinate Reference System (CRS) as the vector object.
		//! @return Number of the added parent polygon element or < 0 if error
		INT32 AddSubRegion (
			const SUBREGION2D& SubRegion
			);
			
		//! Mark an element as deleted
		ERRVALUE DeleteElement (
			ELEMTYPE ElemType,
			INT32 ElemNum
			);
			
		//! Obtain an element iterator to the beginning of the element list
		template <typename _ELEM> ELEMITERATOR<_ELEM> ElementBegin (
			const ELEMITERATOR<_ELEM>& // item
			) const {
			return (ELEMITERATOR<_ELEM>(this));
			}

		//! Obtain an element iterator to the end of the element list
		template <typename _ELEM> ELEMITERATOR<_ELEM> ElementEnd (
			const ELEMITERATOR<_ELEM>& // item
			) const {
			return (ELEMITERATOR<_ELEM>(this, -1));
			}

		//! Locate label style object referenced by 'this'
		//! 'StyleObj' will be attached if one is found, unattached if not
		ERRVALUE FindLabelStyleObject (
			RVC::STYLE& StyleObj
			) const;

		//! Generate the list of islend shell lines for a specific polygon
		ERRVALUE GenerateIsleShellList (
			INT32 ParentPolygon,						//!< Parent polygon, -1 for external polygon shell list
			ORIENTATION orient,
			SIMPLE_ARRAY<INT32>& NumShellLines,
			SIMPLE_ARRAY<INT32>& ShellList,
			SIMPLE_ARRAY<DRECT2D>* RectList = 0
			) const;

		//! Generate nodes and set line start/end values based on the lines in the vector object
		//! This method will remove all existing nodes.
		ERRVALUE GenerateNodes (
			);
		
		//! Generate a point inside of a polygon		
		ERRVALUE GeneratePointInPolygon (
			INT32 PolyNum,
			DPOINT2D& PointInPoly
			) const;
			
		//! Get vector object flags
		//! @return VECTOR::FLAGS
		RVC::VECTOR::FLAGS GetFlags (
			) const;

		//! Obtain style object refered to by the label header
		ERRVALUE GetLabelStyleInfo (
			FILEPATH& filepath, 
			RVC::OBJECTNAME& StyleName
			) const;

		//! Get MAKEPARM values from current vector object instance			
		ERRVALUE GetMakeParms (
			RVC::VECTOR::MAKEPARMS& MakeParms
			) const;
			
		//! Obtain maximum number of islands in a polygon
		//! @return Maximum number of islands
		INT32 GetMaxIslands (
			) const;

		//! Obtain maximum length of a label string
		//! @return Maximum length of a label string
		INT32 GetMaxLabelStringLength (
			) const;

		//! Obtain maximum number of lines in a polygon or node line list
		//! @return Maximum number of lines
		INT32 GetMaxLines (
			) const;

		//! Obtain the maximum number of points a line has
		//! @return Maximum number of points in a line
		INT32 GetMaxPoints (
			) const;

		//! Obtain next available line ID for line ID table
		//! @return Next available line ID
		INT32 GetNextLineID (
			) const;

		//! Obtain next available point ID for point ID table
		//! @return Next available point ID
		INT32 GetNextPointID (
			) const;

		//! Obtain next available polygon ID for polygon ID table
		//! @return Next available polygon ID
		INT32 GetNextPolygonID (
			) const;

		//! Obtain number of labels in the vector object
		//! @return Number of labels
		INT32 GetNumLabels (
			) const;

		//! Obtain number of points in a line
		//! @return Number of line points
		INT32 GetNumLinePoints (
			INT32 LineNum
			) const;

		//! Obtain number of lines in the vector object
		//! @return Number of lines
		INT32 GetNumLines (
			) const;

		//! Obtain number of lines in a node
		//! @return Number of node lines
		INT32 GetNumNodeLines (
			INT32 NodeNum
			) const;

		//! Obtain number of nodes in the vector object
		//! @return Number of nodes
		INT32 GetNumNodes (
			) const;

		//! Obtain number of points in the vector object
		//! @return Number of points
		INT32 GetNumPoints (
			) const;

		//! Obtain number of polygons in the vector object
		//! @return Number of polygons
		INT32 GetNumPolygons (
			) const;

		//! Obtain number of islands in a polygon
		//! @return Number of polygon islands
		INT32 GetNumPolyIslands (
			INT32 PolyNum
			) const;

		//! Obtain number of lines in a polygon
		//! @return Number of polygon lines
		INT32 GetNumPolyLines (
			INT32 PolyNum
			) const;

		//! Obtain vector sub-type
		//! @return Vector object subtype
		SUBTYPE GetSubType (
			) const;

		//! Obtain vector topology type for vector
		//! @return vector RVC::VECTOR::TOPOLOGYTYPE
		TOPOLOGYTYPE GetTopologyType (
			) const;

		//! Obtain limit on the maximum number of vertices a line can have
		//! @return Maximum number of verticies
		INT32 GetVertexLimit (
			) const;

		//! Determine if we are able to generate element ID tables for the vector object
		bool IsElementIDTablesValid (
			) const;

		//! Determine if the two vector objects are equivalent 
		//! This method uses "Fuzzy" vertex comparisons and the direction and order of the elements are not relevant
		//! @return 'True' if they are, 'False' if not
		bool IsEquivalentTo (
			const VECTOR& rhs,
			double threshold
			) const;

		//! Is this point inside the vector polygon specified		
		bool IsPointInPolygon (
			INT32 PolyNum,
			const DPOINT2D& point
			) const;
			
		//! Determine if we are able to generate standard attribute tables for the vector object
		bool IsStdAttribTablesValid (
			) const;
		
		//! Create new vector object given parent object, name, description, creation parameters, and source
		ERRVALUE Make (
			const RVC::OBJECT& parent,
			const DESCRIPTOR& Descriptor,
			const MAKEPARMS& MakeParms,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);
		
		//! Create new vector object given parent object, name, description, creation parameters, and source
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,								//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const MAKEPARMS& MakeParms,
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,
			const CREDENTIALS* Credentials 
#else
			const char* Source = 0,
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);
			
		//! Optimize vector object
		ERRVALUE OptimizeTo (
			RVC::OBJECT& DestParentObj,			//!< Parent object to optimize vector object and copy children to
			RVC::VECTOR& DestVectObj				//!< Destination vector object RETURNED
			);
		
		//! Optimize vector object to specified target location.
		ERRVALUE OptimizeTo (
			RVC::OBJITEM& ObjItemTgt				//!< Target object location passed/returned
			);
		
		//! Read a vector label given a label number
		ERRVALUE Read (
			INT32 LabelNum,
			LABEL& vlabel
			) const;

		//! Read a vector line header given a line number
		ERRVALUE Read (
			INT32 LineNum,
			LINE& vline
			) const;

		//! Read a vector node given a node number
		ERRVALUE Read (
			INT32 NodeNum,
			NODE& vnode
			) const;

		//! Read a vector point given a point number
		ERRVALUE Read (
			INT32 PointNum,
			POINT& vpoint
			) const;

		//! Read a vector polygon given a polygon number
		ERRVALUE Read (
			INT32 PolyNum,
			POLYGON& vpoly
			) const;
			
		ERRVALUE ReadConglomerateLinePoints (
			INT32 StartLineNum,
			POLYLINE& ConglomerateLine,
			CONGLOMERATELINEFILTER& filter
			) const;
			
		//! Read a vector line point
		ERRVALUE ReadLinePoint (
			INT32 LineNum,
			INT32 PointNum,
			DPOINT3D& point
			) const;

		//! Read a vector line point list using the vector POINTYPE into a POLYLINE
		//! The DIMENSION of the POLYLINE will be set to the DIMENSION of the vector object
		//! @return Number of points read or error if < 0
		INT32 ReadLinePoints (
			INT32 LineNum,
			POLYLINE& LinePoints
			) const;

		//! Read a vector line point list using the vector POINTYPE into a POLYLINE
		//! Define 'dimension' to get 2D or 3D lines
		//! @return Number of points read or error if < 0
		INT32 ReadLinePoints (
			INT32 LineNum,
			POLYLINE& LinePoints,
			DIMENSION dimension
			) const;

		//! Read a partial vector line point list into a POLYLINE
		//! @return Number of points read or error if < 0
		INT32 ReadLinePoints (
			INT32 LineNum,
			INT32 Start,
			INT32 Length,
			POLYLINE& LinePoints
			) const;

		//! Read a partial vector line point list into a POLYLINE
		//! Define 'dimension' to get 2D or 3D lines
		//! @return Number of points read or error if < 0
		INT32 ReadLinePoints (
			INT32 LineNum,
			INT32 Start,
			INT32 Length,
			POLYLINE& LinePoints,
			DIMENSION dimension
			) const;

		//! Read a partial vector line point list into a POLYLINE
		//! @return Number of points read or error if < 0
		INT32 ReadLinePoints (
			INT32 LineNum,
			INT32 Start,
			INT32 Length,
			DPOINT2D *pts
			) const;

		//! Read a partial vector line point list into a POLYLINE
		//! Will set the "Extents" optimization in the POLYLINE
		//! @return Number of points read or error if < 0
		INT32 ReadLinePoints (
			INT32 LineNum,
			INT32 Start,
			INT32 Length,
			DPOINT3D *pts
			) const;

		//! Read a vector node line list into a SIMPLE_ARRAY<INT32>
		ERRVALUE ReadNodeLines (
			INT32 NodeNum,
			SIMPLE_ARRAY<INT32>& LineList
			) const;

		//! Read a partial vector node line list into a SIMPLE_ARRAY<INT32>
		ERRVALUE ReadNodeLines (
			INT32 NodeNum,
			INT32 Start,
			INT32 Length,
			SIMPLE_ARRAY<INT32>& LineList
			) const;

		//! Read a vector polygon line list into a SIMPLE_ARRAY<INT32>
		ERRVALUE ReadPolyLines (
			INT32 PolyNum,
			SIMPLE_ARRAY<INT32>& LineList
			) const;

		//! Read a partial vector polygon line list into a SIMPLE_ARRAY<INT32>
		ERRVALUE ReadPolyLines (
			INT32 PolyNum,
			INT32 Start,
			INT32 Length,
			SIMPLE_ARRAY<INT32>& LineList
			) const;

		//! Read a vector polygon island list into a SIMPLE_ARRAY<INT32>
		ERRVALUE ReadPolyIslands (
			INT32 PolyNum,
			SIMPLE_ARRAY<INT32>& IsleList
			) const;

		//! Read a partial vector polygon island list into a SIMPLE_ARRAY<INT32>
		ERRVALUE ReadPolyIslands (
			INT32 PolyNum,
			INT32 Start,
			INT32 Length,
			SIMPLE_ARRAY<INT32>& IsleList
			) const;
		
		//! Read a vector polygon point list using the vector POINTYPE into a POLYLINE
		//! Will set the "Extents" and "Orientation" optimizations in the POLYLINE
		//! Define 'dimension' to get 2D or 3D lines
		ERRVALUE ReadPolyPoints (
			INT32 PolyNum,
			POLYLINE& LinePoints,
			DIMENSION dimension
			) const;

		//! Read a set of vector polygon and possibly its islands into a REGION2D
		ERRVALUE ReadRegion (
			const BITSET_UNOWNED& PolySet,	//!< Set of polygons to generate region from
			REGION2D& Region,						//!< Region to set
			DIMENSION dim,
			BITSET* LineSet = 0,					//!< Set of lines that made up the region, can be NULL
			bool NoIslands = false				//!< 'True' of no islands wanted
			) const;
	
		//! Read a set of vector polygon and possibly its islands into a REGION2D
		ERRVALUE ReadRegion (
			const BITSET_UNOWNED& PolySet,	//!< Set of polygons to generate region from
			REGION2D& Region,						//!< Region to set
			BITSET* LineSet = 0,					//!< Set of lines that made up the region, can be NULL
			bool NoIslands = false				//!< 'True' of no islands wanted
			) const;
	
		//! Read a vector polygon and possibly its islands into a SUBREGION2D
		ERRVALUE ReadSubRegion (
			INT32 PolyNum,				//!< Polygon to generate region from
			SUBREGION2D& SubRegion,
			DIMENSION dim,
			bool NoIslands = false
			) const;
			
		//! Read a vector polygon and possibly its islands into a SUBREGION2D
		ERRVALUE ReadSubRegion (
			INT32 PolyNum,				//!< Polygon to generate region from
			SUBREGION2D& SubRegion,
			bool NoIslands = false
			) const;
			
		//! Set vector object flags
		ERRVALUE SetFlags (
			RVC::VECTOR::FLAGS flags
			);

		//! Set style object refered to by the label header
		ERRVALUE SetLabelStyleInfo (
			const FILEPATH& filepath, 
			const RVC::OBJECTNAME& StyleName
			);
			
		//! Set next available line ID for line ID table
		ERRVALUE SetNextLineID (
			INT32 NewValue
			);
		
		//! Set next available point ID for point ID table
		ERRVALUE SetNextPointID (
			INT32 NewValue
			);
		
		//! Set next available polygon ID for polygon ID table
		ERRVALUE SetNextPolygonID (
			INT32 NewValue
			);
		
		//! Set vector object point coordinate type
		ERRVALUE SetPointType (
			POINTTYPE PointType
			);

		//! Set vector topology type
		ERRVALUE SetTopologyType (
			TOPOLOGYTYPE TopologyType
			);
				
		//! Set vector line vertex limit
		ERRVALUE SetVertexLimit (
			INT32 MaxVertices
			);
				
		//! Write a vector label given a label number
		//! Does not update element nor object extents
		ERRVALUE Write (
			INT32 LabelNum,
			LABEL& vlabel
			);

		//! Write a vector line given a line number
		//! Does not update element nor object extents
		ERRVALUE Write (
			INT32 LineNum,
			const LINE& vline
			);

		//! Write a vector node given a node number
		//! Does not update element nor object extents
		ERRVALUE Write (
			INT32 NodeNum,
			const NODE& vnode
			);

		//! Write a vector point given a point number
		//! Does not update element nor object extents
		ERRVALUE Write (
			INT32 PointNum,
			const POINT& vpoint
			);

		//! Write a vector polygon given a polygon number
		//! Does not update element nor object extents
		ERRVALUE Write (
			INT32 PolyNum,
			const POLYGON& vpoly
			);

		//! Write a vector line point
		//! Does not update element nor object extents
		ERRVALUE WriteLinePoint (
			INT32 LineNum,
			INT32 vertex,
			const DPOINT3D& point
			);

		//! Write a vector line point list
		//! Does not update element nor object extents
		//! @return Number of line points written or error if < 0
		INT32 WriteLinePoints (
			INT32 LineNum,
			const POLYLINE& PointList
			);

		//! Write a partial vector line point list
		//! Does not update element nor object extents
		//! @return Number of line points written or error if < 0
		INT32 WriteLinePoints (
			INT32 LineNum,
			INT32 Start,
			INT32 Length,
			const POLYLINE& PointList
			);

		//! Write a vector node line list from a SIMPLE_ARRAY<INT32>
		//! Does not update element nor object extents
		//! @return Number of node lines written or error if < 0
		INT32 WriteNodeLines (
			INT32 NodeNum,
			const SIMPLE_ARRAY<INT32>& LineList
			);

		//! Write a partial vector node line list from a SIMPLE_ARRAY<INT32>
		//! Does not update element nor object extents
		//! @return Number of node lines written or error if < 0
		INT32 WriteNodeLines (
			INT32 NodeNum,
			INT32 NumLines,
			INT32 Start,
			INT32 Length,
			const SIMPLE_ARRAY<INT32>& LineList
			);

		//! Write a vector polygon line list from a SIMPLE_ARRAY<INT32>
		//! Does not update element nor object extents
		//! @return Number of polygon lines written or error if < 0
		INT32 WritePolyLines (
			INT32 PolyNum,
			const SIMPLE_ARRAY<INT32>& LineList
			);

		//! Write a partial vector polygon line list from a SIMPLE_ARRAY<INT32>
		//! Does not update element nor object extents
		//! @return Number of polygon lines written or error if < 0
		INT32 WritePolyLines (
			INT32 PolyNum,
			INT32 NumLines,
			INT32 Start,
			INT32 Length,
			const SIMPLE_ARRAY<INT32>& LineList
			);

		//! Write a vector polygon island list from a SIMPLE_ARRAY<INT32>
		//! Does not update element nor object extents
		//! @return Number of polygon islands written or error if < 0
		INT32 WritePolyIslands (
			INT32 PolyNum,
			const SIMPLE_ARRAY<INT32>& IsleList
			);

		//! Write a partial vector polygon island list from a SIMPLE_ARRAY<INT32>
		//! Does not update element nor object extents
		//! @return Number of polygon islands written or error if < 0
		INT32 WritePolyIslands (
			INT32 PolyNum,
			INT32 NumIslands,
			INT32 Start,
			INT32 Length,
			const SIMPLE_ARRAY<INT32>& IsleList
			);
			
	protected:

		ERRVALUE ClearLinePointList (
			);

		ERRVALUE ClearNodeLineList (
			);

		ERRVALUE ClearPolygonIslandList (
			);
			
		ERRVALUE ClearPolygonLineList (
			);
			
		ERRVALUE SetNumLabels (
			INT32 NewNumLabels
			);

		ERRVALUE SetNumLines (
			INT32 NewNumLines
			);

		ERRVALUE SetNumNodes (
			INT32 NewNumNodes
			);

		ERRVALUE SetNumPoints (
			INT32 NewNumPolygons
			);

		ERRVALUE SetNumPolygons (
			INT32 NewNumPolygons
			);
			
		ERRVALUE SwapLinePoints (
			INT32 Line1,
			INT32 Line2
			);

		ERRVALUE SwapNodeLines (
			INT32 Node1,
			INT32 Node2
			);

		ERRVALUE SwapPolyIslands (
			INT32 Poly1,
			INT32 Poly2
			);

		ERRVALUE SwapPolyLines (
			INT32 Poly1,
			INT32 Poly2
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		ERRVALUE ExtractLabelsTo (VECTOR& DestObj, const GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg) const;
		ERRVALUE ExtractLinesTo (VECTOR& DestObj, GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractLinesTo (SHAPE& DestObj, GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags, const DBASE_ELEMENT& DBaseObj, const DBTABLE& TableObj) const;
		ERRVALUE ExtractNodesTo (VECTOR& DestObj, const GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractPointsTo (VECTOR& DestObj, GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractPointsTo (SHAPE& DestObj, GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags, const DBASE_ELEMENT& DBaseObj, const DBTABLE& TableObj) const;
		ERRVALUE ExtractPolygonsTo (VECTOR& DestObj, const GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractPolygonsTo (SHAPE& DestObj, GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, GEOMETRIC::EXTRACTFLAGS ExtractFlags, const DBASE_ELEMENT& DBaseObj, const DBTABLE& TableObj) const;

		ERRVALUE ExtractToCAD (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractToRegion (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractToShape (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractToVector (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		
		INT32 FindBestRgnLine (SIMPLE_ARRAY<INT32>& TestList, INT32 LineNum, INT32 NodeNum) const;
		ERRVALUE ReadLabelBaselinePoints (INT32 LabelNum, POLYLINE& Baseline, DIMENSION dim, double Height) const;
		int SetupLabelTransList (const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, RVC::GEOMETRIC::ELEMSELECTFILTER* SelectFilter) const;
		int SetupLineTransList (const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, BITSET& LineSet, RVC::GEOMETRIC::ELEMSELECTFILTER* SelectFilter) const;
		int SetupPointTransList (const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, RVC::GEOMETRIC::ELEMSELECTFILTER* SelectFilter) const;
		int SetupPolygonTransList (const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, BITSET& LineSet, RVC::GEOMETRIC::ELEMSELECTFILTER* SelectFilter) const;
		int TestDatabaseEquivalence (const VECTOR& RhsVectObj, DBASE_ELEMENT& LhsDBaseobj, DBASE_ELEMENT& RhsDBaseobj, SIMPLE_ARRAY<INT32>& TableMatch) const;
			
		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);

		//! Implementation of RVC::GEOSPATIAL virtual methods
		virtual void v_GetExtents (DRECT3D &extents) const;
		virtual void v_GetScale (double& xscale, double& yscale) const;
		virtual double v_GetZScale () const;
		virtual double v_GetZOffset () const;
		virtual bool v_HasCapability (int Capability) const;
		virtual void v_SetScale (double xscale, double yscale);
		virtual void v_SetZOffset (double zoffset);
		virtual void v_SetZScale (double zscale);

		//! Implementation of RVC::GEOMETRIC pure virtual methods
		virtual ERRVALUE v_ExtractTo (RVC::GEOMETRIC& DestObj, RVC::GEOMETRIC::ELEMTRANSLIST& TransList, const TRANS2D_MAPGEN& tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		virtual INT32 v_FindClosestElementLow (ELEMTYPE ElemType, const DPOINT2D& SearchPt, GEOMETRIC::SEARCHFILTER* SearchFilter, double SearchDist, double* DistFromElem) const;
		virtual ERRVALUE v_GetElementExtents (ELEMTYPE ElemType, INT32 ElemNum, DRECT3D& Extents) const;
		virtual INT32 v_GetNextElementID (ELEMTYPE ElemType) const;
		virtual INT32 v_GetNumElements (ELEMTYPE ElemType) const;
		virtual POINTTYPE v_GetPointType () const;
		virtual INT32 v_GetPointSize () const;
		virtual QSEARCHTREE* v_GetSearchTree (ELEMTYPE ElemType) const;
		virtual double v_GetTotalLineLength () const;
		virtual bool v_IsElementDeleted (const RVC::ELEMENT& Element) const;
		virtual bool v_IsEmpty () const;
		virtual ERRVALUE v_SetExtents (const DRECT3D &extents);
		virtual ERRVALUE v_SetNextElementID (ELEMTYPE ElemType, INT32 NextElementID);
		virtual void v_SetSearchTree (ELEMTYPE ElemType, QSEARCHTREE* SearchTree) const;
		virtual ERRVALUE v_SetTotalLineLength (double length);
		virtual int v_SetupElemTransList (const GEOMETRIC::ELEMSELECTPARMS& SelectParms, GEOMETRIC::ELEMTRANSLIST& TransList, GEOMETRIC::ELEMSELECTFILTER *SelectFilter, bool separate) const;
		virtual bool v_TestElementAgainstRegion (const RVC::ELEMENT& Element, const REGION2D& region, TESTCRITERIA TestMode) const;
		virtual ERRVALUE v_UpdateExtents (const DRECT3D &extents);

		//! Implementation of RVC::GEOMETRIC virtual methods
		virtual ERRVALUE v_FindClosestElementsLow (const DPOINT2D& SearchPt, double SearchDist, SEARCHFILTER* SearchFilter, SIMPLE_ARRAY<RVC::ELEMENT>& ElementList, COMPONENT Component) const;
		virtual INT32 v_GetAttachmentIndex (const RVC::ELEMENT& Element) const;
		virtual SPATIALTREE* v_GetSpatialTree () const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, POLYLINELIST& LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, POLYLINE& PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadElement (INT32 ElemNum, REGION2D& Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextLabel (ELEMENT& Element, MILIST<GEOMETRIC::LABEL>& Label, DIMENSION dim, STYLE* StyleObj) const;
		virtual ERRVALUE v_ReadNextLine (ELEMENT& Element, POLYLINELIST& LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextNode (ELEMENT& Element, DPOINT3D& Node) const;
		virtual ERRVALUE v_ReadNextPoint (ELEMENT& Element, POLYLINE& PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextRegion (ELEMENT& Element, REGION2D& Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadRegion (const BITSET_UNOWNED& ElemSet, REGION2D& Region, DIMENSION dim, bool NoIslands) const;
		virtual void v_SetSpatialTree (SPATIALTREE*) const;
		
		//! No values are unchangeable in header

		VECTORHANDLE* m_VectorHandle;
		POINTTYPE m_PointType;					//!< Changed when callback changes it
		TOPOLOGYTYPE m_TopologyType;			//!< Changed when callback changes it
		mutable QSEARCHTREE *m_LabelSearchTree;
		mutable QSEARCHTREE *m_LineSearchTree;
		mutable QSEARCHTREE *m_NodeSearchTree;
		mutable QSEARCHTREE *m_PointSearchTree;
		mutable QSEARCHTREE *m_PolySearchTree;
		mutable SPATIALTREE *m_SpatialTree;
		class OPTIMIZE;
		mutable OPTIMIZE* m_Optimize;
		mutable bool m_SetInstanceData;		//!< This object set the instance data if true, it needs to remove it on closing
		
		class APPINSTANCEDATA;
		friend class APPINSTANCEDATA;
		friend class LABEL;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


class VECTOR::CONGLOMERATELINEFILTER {
	public:
		//! Virtual method to determine if the line is to be conglomerated
		virtual bool Filter (
			const VECTOR& VectObj,
			INT32 TestLineNum
			) = 0;
	};


//! GETOBJFILTER derived class to determine if the vector object has points or lines or polygons to
//! perform geometric operations on.  The RVC::GEOMETRIC::ISNOTEMPTY class will let vector objects with
//! only label elements to be valid.
class VECTOR::GETOBJFILTER_HASGEOMETRICELEMENTS : public GETOBJFILTER {
	public:
		GETOBJFILTER_HASGEOMETRICELEMENTS () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool v_IsSelectable (const RVC::OBJITEM& ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//! GETOBJFILTER derived class to determine if the vector object has the minimum number of labels
class VECTOR::GETOBJFILTER_HASLABELS : public GEOMETRIC::GETOBJFILTER_HASMINIMUMELEMENTS {
	public:
		GETOBJFILTER_HASLABELS (
			INT32 MinLabels = 1
			) : GETOBJFILTER_HASMINIMUMELEMENTS(ELEMTYPE_Label, MinLabels) {}
	};

//! GETOBJFILTER derived class to determine if the vector object has the minimum number of labels
class VECTOR::GETOBJFILTER_HASLINES : public GEOMETRIC::GETOBJFILTER_HASMINIMUMELEMENTS {
	public:
		GETOBJFILTER_HASLINES (
			INT32 MinLines = 1
			) : GETOBJFILTER_HASMINIMUMELEMENTS(ELEMTYPE_Line, MinLines) {}
	};
	
//! GETOBJFILTER derived class to determine if the vector object has the minimum number of labels
class VECTOR::GETOBJFILTER_HASPOINTS : public GEOMETRIC::GETOBJFILTER_HASMINIMUMELEMENTS {
	public:
		GETOBJFILTER_HASPOINTS (
			INT32 MinPoints = 1
			) : GETOBJFILTER_HASMINIMUMELEMENTS(ELEMTYPE_Point, MinPoints) {}
	};
	
//! GETOBJFILTER derived class to determine if the vector object has the minimum number of labels
class VECTOR::GETOBJFILTER_HASPOLYGONS : public GEOMETRIC::GETOBJFILTER_HASMINIMUMELEMENTS {
	public:
		GETOBJFILTER_HASPOLYGONS (
			INT32 MinPolygons = 1
			) : GETOBJFILTER_HASMINIMUMELEMENTS(ELEMTYPE_Polygon, MinPolygons) {}
	};

//! Vector label class
class VECTOR::LABEL {
	private:
		class PRIV;
		
	public:
	
		class LEADERLINE {
			public:

				//! Default constructor
				LEADERLINE (
					);

				//! Copy constructor
				LEADERLINE (
					const LEADERLINE& rhs
					);

				//! Destructor
				~LEADERLINE (
					);

				//! Assignment
				LEADERLINE& operator= (
					const LEADERLINE& rhs
					);

				INT32 GetElemNum (
					) const;

				ELEMTYPE GetElemType (
					) const;

				//! Determine extents of this leader line
				void GetExtents (
					DRECT3D& extents
					) const;
					
				const POLYLINE& GetLeaderPoints (
					) const;
					
				const DPOINT3D& GetTiePoint (
					) const;

				//! Determine if the two label leader lines are equivalent 
				//! This method uses "Fuzzy" vertex comparisons and the direction and order of the elements are not relevant
				//! @return 'True' if they are, 'False' if not
				bool IsEquivalentTo (
					const LEADERLINE& rhs,
					double threshold
					) const;

				void SetElement (
					const RVC::ELEMENT& Element
					);

				void SetElemNum (
					INT32 ElemNum
					);

				void SetElemType (
					ELEMTYPE ElemType
					);

				void SetLeaderPoints (
					const POLYLINE& LeaderPts
					);
					
				void SetTiePoint (
					const DPOINT3D& NewTiePoint
					);

				//! Translate label leader line through a generic coordinate transformation
				ERRVALUE Translate (
					const TRANS2D_MAPGEN& tmg
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				class PRIV;
				PRIV* m_pPriv;

				ERRVALUE Add (VECTORHANDLE* VectorHandle, INT32 LabelNum, INT32 NumLeaderLines, INT32 LeaderLineIndex);

				friend class LABEL;
				friend class LABEL::PRIV;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		typedef MILIST<LEADERLINE>::ITERATOR LEADERLINEITERATOR;
		typedef MILIST<LEADERLINE>::CONST_ITERATOR CONST_LEADERLINEITERATOR;

		//! Default constructor
		LABEL (
			);

		//! Copy constructor
		LABEL (
			const LABEL& rhs
			);

		//! Destructor
		~LABEL (
			);

		//! Assignment
		LABEL& operator= (
			const LABEL& rhs
			);

		//! Add a leader line to the label
		void AddLeaderLine (
			const LEADERLINE& LeaderLine
			);

		//! Clear the label of the string and base points and leader lines
		void Clear (
			);
			
		//! Clear the leader lines of the label
		void ClearLeaderLines (
			);
			
		//! Delete a leader line from the label
		void DeleteLeaderLine (
			MILIST<LEADERLINE>::ITERATOR it
			);

		//! Get label baseline points			
		//! @return Label baseline points
		const POLYLINE& GetBaselinePoints (
			) const;

		//! Compute the extents of this label
		ERRVALUE GetExtents (
			DRECT3D& extents,
			RVC::STYLE* StyleObj = 0
			) const;
			
		//! Compute the extents of this label
		ERRVALUE GetExtents (
			DRECT3D& extents,
			const TEXTSTYLE& TextStyle
			) const;
			
		//! Get the label height in object units
		//! @return Label height
		double GetHeight (
			) const;

		INT32 GetNumLeaderLines (
			) const;
			
		//! Get the label string
		//! @return Label string
		const MISTRING& GetString (
			) const;

		//! Get the label style index
		//! @return Label style index
		INT32 GetStyleIndex (
			) const;
			
		//! Get the label style name
		//! @return Label style name
		const RVC::OBJECTNAME& GetStyleName (
			) const;
			
		//! Does the label follow the baseline?
		//! @return 'True' if label follows baseline
		bool IsBaselineFit (
			) const;
			
		//! Does the label follow the splined baseline?
		//! @return 'True' if label follows the splined baseline
		bool IsBaselineSplineFit (
			) const;
			
		//! Clip vector features under the label when rendering
		//! @return 'True' if features under the label needs to be clipped
		bool IsClipUnderSet (
			) const;
			
		//! Determine if the two label elements are equivalent 
		//! This method uses "Fuzzy" vertex comparisons and the direction and order of the elements are not relevant
		//! @return 'True' if they are, 'False' if not
		bool IsEquivalentTo (
			const LABEL& rhs,
			double threshold
			) const;

		//! Is label valid
		//! @return 'True' if the label is valid
		bool IsValid (
			) const;

		//! Obtain beginning label leader line const iterator
		//! @returns CONST_ITERATOR
		MILIST<LEADERLINE>::CONST_ITERATOR LeaderLineBegin (
			) const;

		//! Obtain beginning label leader line iterator
		//! @returns ITERATOR
		MILIST<LEADERLINE>::ITERATOR LeaderLineBegin (
			);

		//! Obtain ending label leader line const iterator
		//! @returns CONST_ITERATOR
		MILIST<LEADERLINE>::CONST_ITERATOR LeaderLineEnd (
			) const;

		//! Obtain ending label leader line iterator
		//! @returns ITERATOR
		MILIST<LEADERLINE>::ITERATOR LeaderLineEnd (
			);

		//! Set the label to follow the baseline
		void SetBaselineFit (
			bool value
			);
			
		//! Set the label to follow the splined baseline
		void SetBaselineSplineFit (
			bool value
			);
			
		//! Set label baseline points			
		void SetBaselinePoints (
			const POLYLINE& baseline
			);

		//! Set the label clip under flag
		void SetClipUnder (
			bool value
			);
			
		//! Set the label height in object units
		void SetHeight (
			double height
			);

		//! Set the label string
		void SetString (
			const MISTRING& string
			);
			
		void SetStyle (
			const RVC::OBJECTNAME& StyleName,
			INT32 StyleIndex = -1
			);
			
		//! Translate label through a generic coordinate transformation
		ERRVALUE Translate (
			const TRANS2D_MAPGEN& tmg
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PRIV* m_pPriv;

		INT32 Add (VECTORHANDLE* VectorHandle);
		ERRVALUE Read (const VECTOR& VectObj, INT32 LabelNum);
		ERRVALUE Write (VECTORHANDLE* VectorHandle, INT32 LabelNum);

		friend class VECTOR;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

	
//! Vector Line record definition.
struct VECTOR::LINE {
	DPOINT3D minval;							//!< Line minimum extents
	DPOINT3D maxval;							//!< Line maximum extents
	INT32 left, right;						//!< Polygons on left and right side of line, -1 if none
	INT32 start, end;							//!< Nodes at start and end of line, may be equal
	
	LINE (
		):
		minval(0,0,0),
		maxval(0,0,0),
		left(-1),
		right(-1),
		start(-1),
		end(-1)
		{ }

	//! Get extents of vector line.
	//! @return extents of vector line
	DRECT3D GetExtents (
		) const {
		DRECT3D retval(minval);
		retval.Extend(maxval);
		return (retval);
		}
		
	bool IsValid (
		) const { return (!IsNaN(minval.x)); }
		
	void SetExtents (
		const DRECT3D& rect
		) {
		minval.x = rect.xinit;		
		minval.y = rect.yinit;		
		minval.z = rect.zinit;		
		maxval.x = rect.xlast;
		maxval.y = rect.ylast;
		maxval.z = rect.zlast;
		return;
		}
	};


//! Vector Node record definition.
struct VECTOR::NODE : public DPOINT3D {
	NODE () : DPOINT3D(0.0,0.0,0.0) {}
	NODE (const DPOINT2D& rhs) : DPOINT3D(rhs) {}
	NODE (const DPOINT3D& rhs) : DPOINT3D(rhs) {}

	//! Assignment from DPOINT3D
	NODE& operator= (
		const DPOINT3D& rhs
		) { *static_cast<DPOINT3D*>(this) = rhs; return (*this); }

	//! Assignment from DPOINT2D
	NODE& operator= (
		const DPOINT2D& rhs
		) { *static_cast<DPOINT3D*>(this) = rhs; return (*this); }

	//! Get extents of vector node.
	//! @return extents of vector node
	DRECT3D GetExtents (
		) const { return (DRECT3D(*this)); }
		
	bool IsValid (
		) const { return (!IsNaN(x)); }
	};


//! Vector point record definition.
struct VECTOR::POINT : public DPOINT3D {
	POINT () : DPOINT3D(0.0,0.0,0.0) {}
	POINT (const DPOINT2D& rhs) : DPOINT3D(rhs) {}
	POINT (const DPOINT3D& rhs) : DPOINT3D(rhs) {}

	//! Assignment from DPOINT3D
	POINT& operator= (
		const DPOINT3D& rhs
		) { *static_cast<DPOINT3D*>(this) = rhs; return (*this); }

	//! Assignment from DPOINT2D
	POINT& operator= (
		const DPOINT2D& rhs
		) { *static_cast<DPOINT3D*>(this) = rhs; return (*this); }

	//! Get extents of vector point.
	//! @return extents of vector point
	DRECT3D GetExtents (
		) const { return (DRECT3D(*this)); }
		
	bool IsValid (
		) const { return (!IsNaN(x)); }
	};


//! Vector Polygon record definition.
struct VECTOR::POLYGON {
	DPOINT3D minval;							//!< Polygon minimum extents
	DPOINT3D maxval;							//!< Polygon maximum extents
	INT32 polyinside;							//!< Polygon we're inside of, or -1 if not contained in another polygon
	#ifndef GENERATING_DOXYGEN_OUTPUT
	UINT8 space[4];
	#endif //!< GENERATING_DOXYGEN_OUTPUT

	POLYGON (
		):
		minval(0,0,0),
		maxval(0,0,0),
		polyinside(-1)
		{ }

	//! Get extents of vector polygon.
	//! @return extents of vector polygon
	DRECT3D GetExtents (
		) const {
		DRECT3D retval(minval);
		retval.Extend(maxval);
		return (retval);
		}

	bool IsValid (
		) const { return (!IsNaN(minval.x)); }
		
	void SetExtents (
		const DRECT3D& rect
		) {
		minval.x = rect.xinit;		
		minval.y = rect.yinit;		
		minval.z = rect.zinit;		
		maxval.x = rect.xlast;
		maxval.y = rect.ylast;
		maxval.z = rect.zlast;
		return;
		}
	};


//! Vector object creation parameters
class VECTOR::MAKEPARMS : public RVC::GEOMETRIC::MAKEPARMSBASE {
	public:

		//! Default constructor, initialize to 2D Polygonal topology by default
		MAKEPARMS (
			POINTTYPE PointType = POINTTYPE_2DXY,
			TOPOLOGYTYPE TopologyType = TOPOLOGYTYPE_Polygonal,
			FLAGS flags = FLAG_NoElementIDTables
			) :
			GEOMETRIC::MAKEPARMSBASE(PointType),
			m_TopologyType(TopologyType),
			m_Flags(flags),
			m_VertexLimit(0),
			m_NextLineID(1),
			m_NextPointID(1),
			m_NextPolygonID(1)
			{}

		//! Template constructor
		MAKEPARMS (
			const RVC::VECTOR& Template
			);

		//! Combination constructor
		MAKEPARMS (
			const RVC::OBJITEMLIST& GeoObjItemList		//!< Currently 'vector' and 'shape' are the only valid objects
			);

		//! Get vector object flags
		//! @return VECTOR::FLAGS
		FLAGS GetFlags (
			) const {return (m_Flags);}

		//! Obtain next vector line ID value
		//! @return Next vector line ID value
		INT32 GetNextLineID (
			) const {return (m_NextLineID);}

		//! Obtain next vector point ID value
		//! @return Next vector point ID value
		INT32 GetNextPointID (
			) const {return (m_NextPointID);}

		//! Obtain next vector polygon ID value
		//! @return Next vector polygon ID value
		INT32 GetNextPolygonID (
			) const {return (m_NextPolygonID);}

		//! Obtain vector topology type for vector creation
		//! @return vector RVC::VECTOR::TOPOLOGYTYPE
		TOPOLOGYTYPE GetTopologyType (
			) const {return (m_TopologyType);}
			
		//! Obtain vector line vertex limit
		//! @return vector line vertex limit
		INT32 GetVertexLimit (
			) const {return (m_VertexLimit);}

		//! Set vector object flags
		void SetFlags (
			FLAGS flags
			) {m_Flags = flags;}

		//! Set vector line next element ID value			
		void SetNextLineID (
			INT32 NextLineID
			) {m_NextLineID = NextLineID;}
			
		//! Set vector point next element ID value			
		void SetNextPointID (
			INT32 NextPointID
			) {m_NextPointID = NextPointID;}
			
		//! Set vector polygon next element ID value			
		void SetNextPolygonID (
			INT32 NextPolygonID
			) {m_NextPolygonID = NextPolygonID;}
			
		//! Set topology type for vector creation
		void SetTopologyType (
			TOPOLOGYTYPE TopologyType
			) {m_TopologyType = TopologyType;}

		//! Set vector line vertex limit
		void SetVertexLimit (
			INT32 VertexLimit
			) {m_VertexLimit = VertexLimit;}
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		TOPOLOGYTYPE m_TopologyType;
		FLAGS m_Flags;
		INT32 m_VertexLimit;
		INT32 m_NextLineID;
		INT32 m_NextPointID;
		INT32 m_NextPolygonID;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	

//! General operation functions
ERRVALUE GeneratePolygonTableFromLabels (
	const RVC::OBJITEM& ObjItem,
	bool AllowMultiple					//!<  1-Allow multiple label assignments 
	);

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(VECTOR::FLAGS)
#endif

}		// End of RVC namespace

#endif 	// INC_RVC_VECTOR_H

