/**
 * \file imagefiltercrop.h <spatmod/imagefiltercrop.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_CROP definition.
 *
 * \if NODOC
 * $Id: imagefiltercrop.h_v 1.5 2011/03/31 16:04:46 mju Exp $
 *
 * $Log: imagefiltercrop.h_v $
 * Revision 1.5  2011/03/31 16:04:46  mju
 * Add v_getTypeName.
 *
 * Revision 1.4  2011/02/03 16:31:51  mju
 * Add beginIteratorPrevStage and v_queryIterator.
 *
 * Revision 1.3  2007/09/19 15:24:07  mju
 * Add setOrigin.
 *
 * Revision 1.2  2007/04/25 14:03:47  mju
 * Fix class description.
 *
 * Revision 1.1  2006/08/03 15:35:09  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERCROP_H
#define	INC_SPATMOD_IMAGEFILTERCROP_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to crop an image.
class FILTER_CROP : public FILTER {
	public:

		//! Construct with specified rectangle.
		//! Rectangle must be contained within source image or error will be returned by Initialize.
		FILTER_CROP (
			STAGE *pInputImage,						//!< Input image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			const LRECT2D& InputRect				//!< Rectangle in source image coordinates
			);

		//! Destructor.
		virtual ~FILTER_CROP ();

		//! Set origin to new location.
		//! Origin may not result in part of area being outside input stage extents.
		//! Use care if filter is already part of a pipeline as this will result in change of both
		//! georeference and tile sizes.  Total dimensions will remain unchanged.
		ERRVALUE SetOrigin (
			INT32 StartColumn,
			INT32 StartRow
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LRECT2D m_InputRect;

		ERRVALUE BeginIteratorPrevStage (const SECTION_ITERATOR& Iterator, SECTION_ITERATOR& itPrevStage) const;
		void UpdateRect ();

		// STAGE/FILTER overrides.
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		FILTER_CROP (const FILTER_CROP&);
		FILTER_CROP& operator= (const FILTER_CROP&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERCROP_H

