/**
 * \file imagetargetpng.h <spatmod/imagetargetpng.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_PNG definition.
 *
 * \if NODOC
 *
 * $Id: imagetargetpng.h_v 1.6 2011/10/11 14:55:09 mju Exp $
 *
 * $Log: imagetargetpng.h_v $
 * Revision 1.6  2011/10/11 14:55:09  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.5  2009/08/13 16:46:32  mju
 * Base factory not nested.
 *
 * Revision 1.4  2009/08/13 15:38:27  mju
 * Add factory.
 *
 * Revision 1.3  2007/04/11 22:54:52  dfriberg
 * Remove unneeded member
 *
 * Revision 1.2  2007/01/26 23:27:34  dfriberg
 * Add virtual methods to be called when writing one output file per tile
 *
 * Revision 1.1  2005/12/16 22:04:06  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETPNG_H
#define	INC_SPATMOD_IMAGETARGETPNG_H

#ifndef	INC_SPATMOD_IMAGETARGETFILE_H
#include <spatmod/imagetargetfile.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! PNG image target.
//! Supports DATATYPE of UINT8, UINT16 only.
//! Supports PIXEL::TYPE of Single, Grayscale, Indexed, RGB, RGBA only.
//! If Indexed then pixel lookup table must use RGB or RGBA.
class TARGET_PNG : public TARGET_FILE {
	public:

		//! Factory class for constructing TARGET_PNG instance.
		class FACTORY : public FACTORY_TARGET_FILE {
			public:

				//! Default constructor.
				FACTORY ();

				//! Copy constructor.
				FACTORY (const FACTORY& rhs);

				//! Destructor.
				virtual ~FACTORY ();

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				// FACTORY_TARGET_FILE overrides.
				virtual FACTORY_TARGET_FILE * v_Clone () const;
				virtual TARGET_FILE * v_ConstructTarget (STAGE *pInputStage, const FILEPATH& filepath, GEOFORMATS GeoFormats) const;

				// Unimplemented.
				FACTORY& operator= (const FACTORY& rhs);
				#endif
			};

		//! Construct target with specified path.
		TARGET_PNG (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const FILEPATH& filepath,					//!< Path to target file
			GEOFORMATS GeoFormats = GEOFORMAT_None	//!< Georeference formats
			);

		//! Destructor.
		virtual ~TARGET_PNG ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class FILE_PNG;
		FILE_PNG *m_pFile;
		SIMPLE_ARRAY<UINT8> m_RowBuf;

		ERRVALUE MakeFile(const FILEPATH&, const LRECT2D& extents);
		// TARGET overrides.
		virtual ERRVALUE v_FillTileRows ( INT32 NumColumns, INT32 NumRows);
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();
		virtual ERRVALUE v_TileBegin (const	FILEPATH& filepath,const LRECT2D& extents);
		virtual ERRVALUE v_TileEnd ();
		virtual ERRVALUE v_WriteSubSection (const SECTION& SubSection, const BUFFER& buffer);

		// Unimplemented
		TARGET_PNG (const TARGET_PNG&);
		TARGET_PNG& operator= (const TARGET_PNG&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETPNG_H

