/**
 * \file spatmod/imagetargettilesetkml.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_TILESET_KML definition.
 *
 * \if NODOC
 * $Id: imagetargettilesetkml.h_v 1.28 2012/04/20 17:00:21 bcolombe Exp $
 *
 * $Log: imagetargettilesetkml.h_v $
 * Revision 1.28  2012/04/20 17:00:21  bcolombe
 * Add server url for making aux files
 *
 * Revision 1.27  2011/10/11 15:02:24  mju
 * Remove textid hdr.
 *
 * Revision 1.26  2011/06/20 20:27:54  bcolombe
 * Add CreateFullStructure
 *
 * Revision 1.25  2011/05/24 20:40:16  ldyas
 * added parameter to allow level to affect leaf minlodpixels
 *
 * Revision 1.24  2011/04/20 23:21:03  bcolombe
 * Added ResampleMethod for reduced levels
 *
 * Revision 1.23  2011/04/14 16:43:12  bcolombe
 * Remove extents from ctor, determine from input
 *
 * Revision 1.22  2011/03/29 15:45:42  bcolombe
 * Add apikey to createhtml
 *
 * Revision 1.21  2010/11/01 22:46:07  bcolombe
 * Imgformat changes
 *
 * Revision 1.20  2010/07/20 21:56:09  bcolombe
 * Add options for how to process kmls
 *
 * Revision 1.19  2010/07/06 22:30:59  bcolombe
 * Add WriteBranchKML and WriteLeafKML that take tile number
 *
 * Revision 1.18  2010/06/22 21:53:58  bcolombe
 * Added ResolveBranchKML
 *
 * Revision 1.17  2010/06/03 23:12:44  bcolombe
 * Added UsePalette to ComputeReduced
 *
 * Revision 1.16  2010/06/01 20:59:30  bcolombe
 * Added WriteRootKML that takes tile location
 *
 * Revision 1.15  2010/05/18 16:44:56  bcolombe
 * Add ComputeReducedLevel that takes source & target locations
 *
 * Revision 1.14  2010/04/20 23:27:38  bcolombe
 * Added zipoutput to ComputeReducedLevel
 *
 * Revision 1.13  2010/04/19 16:32:26  bcolombe
 * Added bOverwriteExisting to CreateHTML and WriteRootKML
 *
 * Revision 1.12  2010/04/07 23:13:08  bcolombe
 * Pass tileset to CreateHTML
 *
 * Revision 1.11  2010/04/02 22:25:45  bcolombe
 * Make CreateHTML, WriteRootKML and ComputeReducedLevel static
 *
 * Revision 1.10  2010/03/31 22:37:39  bcolombe
 * Add MinLevel to WriteRootKML
 *
 * Revision 1.9  2010/03/18 16:14:51  bcolombe
 * Added ctor
 *
 * Revision 1.8  2009/12/03 21:57:06  bcolombe
 * Added optional message to ComputeReducedLevel
 *
 * Revision 1.7  2009/11/04 23:47:26  bcolombe
 * Constructor takes basepath instead of kmlpath
 *
 * Revision 1.6  2009/11/04 21:29:37  bcolombe
 * Changed description of filepath in constructor
 *
 * Revision 1.5  2009/08/25 13:26:29  mju
 * Don't override generateTileFilePath.
 *
 * Revision 1.4  2009/08/13 17:52:27  bcolombe
 * WriteRootKML is now public
 *
 * Revision 1.3  2009/08/07 17:54:07  mju
 * Add setGeogExtents.
 *
 * Revision 1.2  2009/08/06 17:01:57  mju
 * *** empty log message ***
 *
 * Revision 1.1  2009/08/05 17:43:12  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETTILESETKML_H
#define	INC_SPATMOD_IMAGETARGETTILESETKML_H

#ifndef	INC_SPATMOD_IMAGETARGETILESETFILE_H
#include <spatmod/imagetargettilesetfile.h>
#endif

class MISTATUSCONTEXT;

namespace SPATMOD {
namespace IMAGE {

class TARGETPARM_TIFF;

//=====================================================================================================================

//! KML target.
class TARGET_TILESET_KML : public TARGET_TILESET_FILE {
	public:

		//! Create GoogleMaps HTML file
		static ERRVALUE CreateHTML (
			TILESET& Tileset,				//!< Tileset to use
			const MISTRING& ServerURL,		//!< Server location where html will be
			bool bOverwriteExisting = true,	//!< Write over any existing files
			MISTRING GoogleAPIKey = TEXTID__None			//!< Google API key
			);

		//! Create GoogleMaps HTML file
		static ERRVALUE CreateHTML (
			TILESET& Tileset,				//!< Tileset to use
			bool bOverwriteExisting = true,	//!< Write over any existing files
			MISTRING GoogleAPIKey = TEXTID__None			//!< Google API key
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			INT32 level,										//!< Level to compute
			MISTRING message = TEXTID_Creating_ResolutionLevelN	//!< Context message to display
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			const FILEPATH& SourcePath,							//!< Location of source tiles
			const FILEPATH& TargetPath,							//!< Location to output tiles
			INT32 level,										//!< Level to compute
			MISTRING message = TEXTID_Creating_ResolutionLevelN	//!< Context message to display
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			INT32 level,										//!< Level to compute
			TILESET::IMGFORMAT ImgFormat = TILESET::IMGFORMAT_Invalid,	//!< ImgFormat
			UINT16 QualityJPEG = 75,							//!< Quality of JPEG output
			TARGETPARM_TIFF *pParmsTIFF = NULL,					//!< Tiff settings
			MISTRING message = TEXTID_Creating_ResolutionLevelN,//!< Context message to display
			bool bZipOutput = false								//!< Zip results
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			const FILEPATH& SourcePath,							//!< Location of source tiles
			const FILEPATH& TargetPath,							//!< Location to output tiles
			INT32 level,										//!< Level to compute
			TILESET::IMGFORMAT ImgFormat,						//!< ImgFormat
			const IMAGE::PIXEL& PixelProperties,				//!< Pixel Properties
			UINT16 QualityJPEG = 75,							//!< Quality of JPEG output
			TARGETPARM_TIFF *pParmsTIFF = NULL,					//!< Tiff settings
			MISTRING message = TEXTID_Creating_ResolutionLevelN,//!< Context message to display
			bool bZipOutput = false,								//!< Zip results
			RESAMPLEMETHOD ResampleMethod = RESAMPLEMETHOD_Nearest	//!< Resample Method to use for zoom filter
			);

		//! Compute level for super-overlay structure with links to level + 1
		static ERRVALUE ComputeStructureLevel (
			TILESET& Tileset,									//!< Tileset to use
			const FILEPATH& SourcePath,							//!< Location of source tiles
			const FILEPATH& TargetPath,							//!< Location to output tiles
			INT32 level,										//!< Level to compute
			MISTRING message = TEXTID_Creating_ResolutionLevelN,//!< Context message to display
			bool bZipOutput = false								//!< Zip results
			);

		//! Create a branch kml
		static ERRVALUE WriteBranchKML (
			TILESET& Tileset,					//!< Tileset to use
			const FILEPATH& TileLocation,		//!< Location of tiles
			INT32 level,						//!< Zoom level
			const LPOINT2D& TileIdx,			//!< Absolute tile index within defined tile range for zoom level
			bool bPutGroundOverlay = true		//!< Include ground overlay
			);

		//! Create a leaf kml
		static ERRVALUE WriteLeafKML (
			TILESET& Tileset,					//!< Tileset to use
			const FILEPATH& TileLocation,		//!< Location of tiles
			INT32 level,						//!< Zoom level
			const LPOINT2D& TileIdx				//!< Absolute tile index within defined tile range for zoom level
			);

		//! Create the root kml
		static ERRVALUE WriteRootKML (
			TILESET& Tileset,					//!< Tileset to use
			const MISTRING& ServerURL,			//!< Server location where kml will be
			const FILEPATH& KMLLocation,		//!< Location to write KML
			const FILEPATH& TileLocation,		//!< Location of tiles
			bool bOverwriteExisting = true,		//!< Write over any existing files
			INT32 MinLevel = -1					//!< Minimum zoom level to include
			);

		//! Create the root kml
		static ERRVALUE WriteRootKML (
			TILESET& Tileset,					//!< Tileset to use
			const FILEPATH& KMLLocation,		//!< Location to write KML
			const FILEPATH& TileLocation,		//!< Location of tiles
			bool bOverwriteExisting = true,		//!< Write over any existing files
			INT32 MinLevel = -1					//!< Minimum zoom level to include
			);

		//! Produce branch kmls for all upper levels
		static ERRVALUE ResolveBranchKML (
			TILESET& Tileset,					//!< Tileset to use
			const FILEPATH& TileLocation,		//!< Location of tiles
			bool bOverwriteExisting = true,		//!< Write over any existing files
			INT32 MinLevel = -1					//!< Minimum zoom level to include
			);

		//! Constructor.
		TARGET_TILESET_KML (
			STAGE *pInputStage,						//!< Stage in pipeline to obtain image data from
			TILESET::IMGFORMAT format,					//!< Image tile file format
			const FILEPATH& filepath,				//!< Basepath used for creating TSD and KML files
			INT32 TileSize,							//!< Tile size
			IMAGE::TILESET::PROFILE profile			//!< Tileset profile
			);

		//! Constructor.  Will create on global grid.
		TARGET_TILESET_KML (
			STAGE *pInputStage,						//!< Stage in pipeline to obtain image data from
			TILESET::IMGFORMAT format,					//!< Image tile file format
			const FILEPATH& filepath,				//!< Basepath used for creating TSD and KML files
			INT32 TileSize,							//!< Tile size
			INT32 MinLevel,							//!< Minimum zoom level to compute
			INT32 MaxLevel							//!< Maximum zoom level to compute
			);

		//! Destructor.
		virtual ~TARGET_TILESET_KML ();

		//! Set geographic (lat-lon) extents for overall result.
		//! If not used extents will be derived from total target image, which may include blank areas around outer tiles.
		void SetGeogExtents (
			const DRECT2D& GeogExtents
			) { m_GeogExtents = GeogExtents; }

		//! Set parameters specific to JPEG format.
		void SetParmsJPEG (
			INT16 quality
			) { m_QualityJPEG = quality; }

		//! Set parameters specific to TIFF format.
		void SetParmsTIFF (
			const TARGETPARM_TIFF& Parm
			);

		//! When writing tiles, process the kmls as branch
		void SetProcessBranchKML (
			bool bProcessBranchKML
			) { m_bProcessBranchKML = bProcessBranchKML; }

		//! When writing tiles, process the kmls as leaf
		void SetProcessLeafKML (
			bool bProcessLeafKML
			) { m_bProcessLeafKML = bProcessLeafKML; }

		//! Create entire super-overlay structure
		void SetCreateFullStructure (
			bool bCreateFullStructure
			) { m_bCreateFullStructure = bCreateFullStructure; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef TARGET_TILESET_FILE BASECLASS;

		class IMAGEINFOKML;

		DRECT2D m_GeogExtents;
		TARGETPARM_TIFF *m_pParmsTIFF;
		UINT16 m_QualityJPEG;
		FILEPATH m_basepath;
		bool m_bProcessBranchKML;
		bool m_bProcessLeafKML;
		bool m_bCreateFullStructure;
		SPATMOD::IMAGE::TILESET m_ProcessTileset;
		MISTATUSCONTEXT *m_pStatusContext;

		static ERRVALUE WriteBranchKML (
			const FILEPATH& PathImage,
			const DRECT2D& GeogExtent,
			const IMAGEINFOKML ImagePrev[2][2],
			INT32 MinLodPixelsImage,
			INT32 MaxLodPixelsImage,
			INT32 level,
			bool bPutGroundOverlay = true
			);

		ERRVALUE WriteLeafKML (
			const FILEPATH& PathImage,
			const SOURCE_MEMORY& SourceTile,
			INT32 level
			);

		// TARGET overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();

		// TARGET_TILESET_FILE overrides.
		ERRVALUE v_CreateTargetFile (IMAGE::STAGE *pStageInput, const FILEPATH& filepath, const IMAGE::FORMAT& format, IMAGE::TARGET_FILE *& pTargetFile) const;
		virtual ERRVALUE v_WriteTile (SOURCE_MEMORY& SourceTile, const LPOINT3D& TileIdx, bool HasNullCells);

		// Unimplemented
		TARGET_TILESET_KML (const TARGET_TILESET_KML&);
		TARGET_TILESET_KML& operator= (const TARGET_TILESET_KML&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETTILESETKML_H

